/*! Tripetto Runner Foundation 5.0.0 - Copyright (C) 2021 Tripetto B.V. - All Rights Reserved */

/// <reference path="tripetto.vfsm.d.ts" />
/// <reference path="tripetto.map.d.ts" />
/// <reference path="tripetto.slots.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />
/// <reference path="marblecore.markdown.d.ts" />
/// <reference path="marblecore.callback.d.ts" />
/// <reference path="marblecore.functional.d.ts" />
/// <reference path="tripetto.contracts.d.ts" />
/// <reference path="marblecore.sha2.d.ts" />
/// <reference path="marblecore.aes.d.ts" />
/// <reference path="marblecore.l10n.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />

declare module "tripetto-runner-foundation" {
    /** Contains the package name. */
    export const NAME: string;
    /** Contains the version number. */
    export const VERSION: string;
    /** Explicit exports */
    export {
        Context,
        Data,
        Errors,
        IDataValues,
        IMoment,
        Instance,
        ISnapshot,
        IVariable,
        Value,
        IOntologyResumeEvent as IRunnerFoundationResumeEvent,
    } from "@tripetto/vfsm";
    export {
        ICondition,
        IConditionBlock,
        IDefinition,
        INode,
        INodeBlock,
        IPrologue,
    } from "@tripetto/map";
    export { Kinds, Slot } from "@tripetto/slots";
    export { TSerializeTypes } from "@marblecore/serializer";
    export { Condition } from "tripetto-runner-foundation/lib/conditions/condition";
    export { ConditionBlock } from "tripetto-runner-foundation/lib/conditions/blocks/block";
    export { ConditionBlocksNamespace } from "tripetto-runner-foundation/lib/conditions/blocks/blocks";
    export {
        IConditionBlockType,
        TConditionBlock,
    } from "tripetto-runner-foundation/lib/conditions/blocks/type";
    export { Branch } from "tripetto-runner-foundation/lib/branches/branch";
    export { Cluster } from "tripetto-runner-foundation/lib/clusters/cluster";
    export { RunnerFoundation } from "tripetto-runner-foundation/lib/runner";
    export {
        IRunnerFoundationChangeEvent,
        IRunnerFoundationRestartEvent,
    } from "tripetto-runner-foundation/lib/events";
    export { Node } from "tripetto-runner-foundation/lib/nodes/node";
    export { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    export { HeadlessBlock } from "tripetto-runner-foundation/lib/nodes/blocks/headless";
    export { NodeBlocksNamespace } from "tripetto-runner-foundation/lib/nodes/blocks/blocks";
    export {
        INodeBlockType,
        TNodeBlock,
    } from "tripetto-runner-foundation/lib/nodes/blocks/type";
    export { IObservableNode } from "tripetto-runner-foundation/lib/storyline/observable";
    export { TChanges } from "tripetto-runner-foundation/lib/changes";
    export { TModes } from "tripetto-runner-foundation/lib/modes";
    export { TStatus } from "tripetto-runner-foundation/lib/status";
    export {
        markdownify,
        markdownifyTo,
        markdownifyToString,
        markdownifyToPlainText,
        markdownifyToURL,
    } from "tripetto-runner-foundation/lib/markdown";
    export { MarkdownFeatures, MarkdownTypes } from "@marblecore/markdown";
    export { isVariable } from "tripetto-runner-foundation/lib/variables";
    export { Storyline } from "tripetto-runner-foundation/lib/storyline/storyline";
    export { IStoryline } from "tripetto-runner-foundation/lib/storyline/interface";
    export { Moment } from "tripetto-runner-foundation/lib/storyline/moment";
    export { IPage } from "tripetto-runner-foundation/lib/storyline/page";
    export { Import } from "tripetto-runner-foundation/lib/data/import";
    export { Export } from "tripetto-runner-foundation/lib/data/export";
    export { IEpilogueContext as IEpilogue } from "tripetto-runner-foundation/lib/epilogue";
    export {
        Namespaces,
        mountNamespace,
        unmountNamespace,
    } from "tripetto-runner-foundation/lib/namespaces";
    /** Decorators */
    export {
        tripetto,
        tripetto as block,
    } from "tripetto-runner-foundation/lib/decorators/block";
    export {
        condition,
        IConditionHandler,
        ConditionResult,
    } from "tripetto-runner-foundation/lib/decorators/condition";
    export {
        validator,
        IValidatorHandler,
        ValidatorResult,
    } from "tripetto-runner-foundation/lib/decorators/validate";
    export {
        destroy,
        IDestroyHandler,
    } from "tripetto-runner-foundation/lib/decorators/destroy";
    export {
        fingerprint,
        calculateFingerprintAndStencil,
    } from "tripetto-runner-foundation/lib/fingerprint";
    export { stencil } from "tripetto-runner-foundation/lib/data/stencil";
    export { checksum } from "tripetto-runner-foundation/lib/data/checksum";
    export {
        powSolve,
        powVerify,
        powHashRate,
        powSpentTime,
        powDuration,
    } from "tripetto-runner-foundation/lib/data/pow";
    /** Implicit exports */
    export * from "@marblecore/callback";
    export * from "@marblecore/functional";
    export * from "@tripetto/contracts";
    /** Namespace exports */
    export * as Slots from "@tripetto/slots";
    export * as SHA2 from "@marblecore/sha2";
    export * as AES from "@marblecore/aes";
    export * as L10n from "@marblecore/l10n";
}

declare module "tripetto-runner-foundation/lib/conditions/condition" {
    /** Dependencies */
    import {
        Condition as ConditionFactory,
        Conditions,
        Context,
    } from "@tripetto/vfsm";
    import { Callback } from "@marblecore/callback";
    import { RunnerFoundation } from "tripetto-runner-foundation/lib/runner";
    import { ConditionResult } from "tripetto-runner-foundation/lib/decorators/condition";
    import { ICondition } from "@tripetto/map";
    export class Condition extends ConditionFactory<ICondition> {
        /**
         * Creates a new condition instance.
         * @param pConditions Reference to the conditions dispatcher.
         * @param pCondition Reference to the condition properties.
         */
        constructor(pConditions: Conditions, pCondition: ICondition);
        /** Retrieves the identifier of the condition. */
        get id(): string;
        /** Reference to the runner. */
        get runner(): RunnerFoundation | undefined;
        /**
         * Invoked when the runner wants to verify the condition.
         * @param pContext Reference to the context.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously. Returns `false` when no
         * block is available.
         * @event
         */
        onCondition(
            pContext: Context,
            pCallback: Callback<boolean>
        ): ConditionResult;
        /**
         * Invoked when the condition is approved.
         * @param pContext Reference to the context.
         * @event
         */
        onApproved(pContext: Context): void;
    }
}

declare module "tripetto-runner-foundation/lib/conditions/blocks/block" {
    /** Dependencies */
    import {
        IBranch,
        ICluster,
        ICondition,
        IConditionBlock,
        INode,
    } from "@tripetto/map";
    import { Condition } from "tripetto-runner-foundation/lib/conditions/condition";
    import { Context, IVariable, ImmutableValue } from "@tripetto/vfsm";
    import { IConditionBlockType } from "tripetto-runner-foundation/lib/conditions/blocks/type";
    import { Kinds, Slot, Slots } from "@tripetto/slots";
    import { TSerializeTypes } from "@marblecore/serializer";
    export abstract class ConditionBlock<Properties extends {} = {}> {
        /** Contains the type identifier of the block. */
        static readonly identifier: string;
        /** Retrieves the block type. */
        readonly type: IConditionBlockType<ConditionBlock<Properties>>;
        /** Reference to the context. */
        readonly context: Context;
        /**
         * Creates a new condition block instance.
         * @param pCondition Reference to the condition.
         * @param pContext Reference to the context.
         */
        constructor(pCondition: Condition, pContext: Context);
        /** Retrieves the current view of the runner. */
        get view(): "live" | "test" | "preview";
        /** Retreieves the condition props. */
        get condition(): Readonly<ICondition>;
        /** Retrieves the block props. */
        get props(): Readonly<Properties & IConditionBlock>;
        /** Retrieves the node props. */
        get node(): Readonly<INode> | undefined;
        /** Retrieves the cluster props. */
        get cluster(): Readonly<ICluster> | undefined;
        /** Retrieves the branch props. */
        get branch(): Readonly<IBranch> | undefined;
        /** Retrieves a reference to the slots (if the condition is connected to a node). */
        get slots(): Slots | undefined;
        /**
         * Retrieves a slot.
         * @param sId Specifies the slot identifier. If omitted the function
         * tries to find the slot that is attached to the condition block.
         * @return Returns the slot or `undefined` if the slot was not found.
         */
        protected slotOf<S extends Slot = Slot>(sId?: string): S | undefined;
        /**
         * Retrieves an immutable value. You can only retrieve values for the
         * slots of an attached node. If you want to retrieve values from other nodes
         * or for conditions without an attached node, use `immutableValueOf`.
         * @param pReference Optional parameter with a slot reference. If
         * omitted the function tries to find the slot that is attached to the
         * condition block.
         * @param sKind Specifies the slot kind.
         * @return Returns the mutable value or `undefined` if the value was not found.
         */
        protected valueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(
            pReference?: Slot | string,
            sKind?: Kinds
        ): ImmutableValue<T, S> | undefined;
        /**
         * Retrieves an immutable value. You can retrieve immutable values for any node.
         * @param pReference Optional parameter with a slot instance or identifier. If
         * omitted the function tries to find the slot that is attached to the
         * condition block.
         * @return Returns the immutable value or `undefined` if the value was not found.
         */
        protected immutableValueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(pReference?: Slot | string): ImmutableValue<T, S> | undefined;
        /**
         * Retrieves a variable.
         * @param pReference Specifies the slot reference, value or variable identifier.
         * @return Returns the variable or `undefined` if the variable was not found.
         */
        protected variableFor(
            pReference:
                | Slot
                | {
                      slot: Slot;
                  }
                | string
        ): IVariable | undefined;
        /**
         * Parses the supplied string, replacing all variables with their current values.
         * @param sString Specifies the string to parse.
         * @param sPlaceholder Specifies a string for empty variables (disabled by default).
         * @param bLineBreaks Specifies if line breaks are supported (disabled by default).
         * @return Returns the parsed string.
         */
        protected parseVariables(
            sString: string,
            sPlaceholder?: string,
            bLineBreaks?: boolean
        ): string;
    }
}

declare module "tripetto-runner-foundation/lib/conditions/blocks/blocks" {
    /** Dependencies */
    import { Condition } from "tripetto-runner-foundation/lib/conditions/condition";
    import { ConditionBlock } from "tripetto-runner-foundation/lib/conditions/blocks/block";
    import { Context } from "@tripetto/vfsm";
    import { TConditionBlock } from "tripetto-runner-foundation/lib/conditions/blocks/type";
    export class ConditionBlocksNamespace {
        /** Creates a new namespace instance. */
        constructor(sIdentifier: string);
        /** Retrieves the identifier for the namespace. */
        get identifier(): string;
        /**
         * Registers a block.
         * @param pBlock Specifies the block.
         * @param sIdentifier Specifies the block type identifier.
         * @param pAliases Specifies type aliases for the block.
         */
        register(
            pBlock: TConditionBlock,
            sIdentifier: string,
            pAliases: string[]
        ): void;
        /**
         * Provides a new instance of a condition block for the supplied condition.
         * @param pCondition Reference to the condition.
         * @param pContext Reference to the context.
         * @return Returns a condition block instance.
         */
        create<T extends ConditionBlock>(
            pCondition: Condition,
            pContext: Context
        ): T | undefined;
    }
}

declare module "tripetto-runner-foundation/lib/conditions/blocks/type" {
    /** Dependencies */
    import { Condition } from "tripetto-runner-foundation/lib/conditions/condition";
    import { ConditionBlock } from "tripetto-runner-foundation/lib/conditions/blocks/block";
    import { Context } from "@tripetto/vfsm";
    export interface IConditionBlockType<T extends ConditionBlock> {
        /** Contains the block type identifier. */
        readonly identifier: string;
        /**
         * Creates a new block instance.
         * @param pCondition Reference to the condition.
         * @param pContext Reference to the context.
         */
        new (pCondition: Condition, pContext: Context): T;
    }
    export type TConditionBlock = IConditionBlockType<ConditionBlock>;
}

declare module "tripetto-runner-foundation/lib/branches/branch" {
    /** Dependencies */
    import { Branch as BranchFactory, Context } from "@tripetto/vfsm";
    import { IBranch } from "@tripetto/map";
    import { RunnerFoundation } from "tripetto-runner-foundation/lib/runner";
    import { Cluster } from "tripetto-runner-foundation/lib/clusters/cluster";
    export class Branch extends BranchFactory<IBranch> {
        /** Retrieves the identifier of the branch. */
        get id(): string;
        /** Reference to the runner. */
        get runner(): RunnerFoundation | undefined;
        /** Reference to the cluster. */
        get cluster(): Cluster;
        /**
         * Invoked when the branch condition is requested.
         * @return For now, the branch condition is always true.
         * @event
         */
        onCondition(): boolean;
        /**
         * Invoked when the branch is left.
         * @param pContext Reference to the context.
         * @event
         */
        onLeave(pContext: Context): void;
    }
}

declare module "tripetto-runner-foundation/lib/clusters/cluster" {
    /** Dependencies */
    import { Await } from "@marblecore/callback";
    import { RunnerFoundation } from "tripetto-runner-foundation/lib/runner";
    import { Context, State } from "@tripetto/vfsm";
    import { ICluster } from "@tripetto/map";
    export class Cluster extends State<ICluster> {
        /** Retrieves the identifier of the cluster. */
        get id(): string;
        /** Reference to the runner. */
        get runner(): RunnerFoundation;
        /** Specifies the position of the cluster in the runner as a percentage. */
        get percentageInRunner(): number;
        /**
         * Retrieves the validation result for the supplied context.
         * @param pContext Reference to the context.
         */
        getValidation(pContext: Context): "unknown" | "pass" | "fail";
        /**
         * Invoked when the enter condition for the cluster is requested.
         * @event
         */
        onEnterCondition(): boolean;
        /**
         * Invoked when the cluster is entered.
         * @param pContext Reference to the context.
         * @param pAwait Reference to the await pointer
         * @return Returns the await reference.
         * @event
         */
        onEnter(pContext: Context, pAwait: Await): Await;
        /**
         * Invoked when the cluster leave condition is requested.
         * @event
         */
        onLeaveCondition(): boolean;
        /**
         * Invoked when the cluster is left (only for paginated runners).
         * @param pContext Reference to the context.
         * @event
         */
        onLeave(pContext: Context): void;
        /**
         * Invoked when the cluster is canceled (only for paginated runners).
         * @param pContext Reference to the context.
         * @event
         */
        onCancel(pContext: Context): void;
        /**
         * Invoked when the cluster needs to be validated.
         * @event
         */
        onValidate(pContext: Context): boolean;
        /**
         * Invoked when the cluster is validated.
         * @param pContext Reference to the context.
         * @param sResult Specifies the validation state.
         * @param sType Specifies if the validation type.
         * @event
         */
        onValidated(
            pContext: Context,
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate"
        ): void;
        /**
         * Invoked when the cluster starts or stops evaluating.
         * @param pContext Reference to the context.
         * @param bEvaluating Specifies if the cluster is evaluating.
         * @event
         */
        onEvaluate(pContext: Context, bEvaluating: boolean): void;
    }
}

declare module "tripetto-runner-foundation/lib/runner" {
    /** Dependencies */
    import { Branch } from "tripetto-runner-foundation/lib/branches/branch";
    import { IDefinition, IPrologue } from "@tripetto/map";
    import { Namespace as L10nNamespace } from "@marblecore/l10n";
    import { Cluster } from "tripetto-runner-foundation/lib/clusters/cluster";
    import { Condition } from "tripetto-runner-foundation/lib/conditions/condition";
    import {
        Context,
        ISnapshot,
        IValueChangeEvent,
        Instance,
        Ontology,
    } from "@tripetto/vfsm";
    import {
        IRunnerFoundationChangeEvent,
        TRunnerFoundationEvents,
    } from "tripetto-runner-foundation/lib/events";
    import { Await } from "@marblecore/callback";
    import { IStoryline } from "tripetto-runner-foundation/lib/storyline/interface";
    import { Node } from "tripetto-runner-foundation/lib/nodes/node";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    import { Storyline } from "tripetto-runner-foundation/lib/storyline/storyline";
    import { TChanges } from "tripetto-runner-foundation/lib/changes";
    import { TModes } from "tripetto-runner-foundation/lib/modes";
    import { TStatus } from "tripetto-runner-foundation/lib/status";
    import { IEpilogueContext } from "tripetto-runner-foundation/lib/epilogue";
    import { INamespace } from "tripetto-runner-foundation/lib/namespaces";
    export class RunnerFoundation<
        T extends NodeBlock = NodeBlock,
        Data = undefined
    > extends Ontology<
        Cluster,
        Node<T>,
        Branch,
        Condition,
        IDefinition,
        TRunnerFoundationEvents
    > {
        /**
         * Invoked when the runner instance is changed.
         * @event
         */
        onChange?: (pEvent: IRunnerFoundationChangeEvent<T>) => void;
        /**
         * Invoked when the collected data of an instance is changed.
         * @event
         */
        onData?: (
            pInstance: Instance,
            pValueChangeEvent: IValueChangeEvent
        ) => void;
        /**
         * Invoked when the runner instance finishes.
         * @event
         */
        onFinish?: (
            pInstance: Instance
        ) => Promise<string | undefined> | boolean;
        /**
         * Invoked when the runner is restarted.
         * @event
         */
        onRestart?: (pInstance: Instance) => void;
        /** Retrieves an initial storyline. */
        static getInitialStoryline<T extends NodeBlock = NodeBlock>(
            sMode: TModes
        ): IStoryline<T>;
        /**
         * Creates a new runner instance.
         * @param pProps Specifies the properties for the runner instance.
         */
        constructor(pProps: {
            /** Specifies the definition to load. */
            readonly definition: IDefinition | string;
            /**
             * Specifies the mode of operation. Can be one of the following values:
             * - `paginated`: Render each cluster on a separate page (this is the default behavior);
             * - `continuous`: Render all completed clusters and the current cluster on a page;
             * - `progressive`: Render all completed, current and future clusters on a page.
             */
            readonly mode: TModes;
            /** Specifies a namespace identifier for the runner. */
            readonly namespace?: string;
            /** Specifies the l10n namespace to use. */
            readonly l10n?: L10nNamespace;
            /** Specifies if the runner should be started. */
            readonly start?: boolean;
            /** Specifies if the runner should run in preview mode (this will start the runner when the `start`-prop is omitted). */
            readonly preview?: boolean;
            /** Specifies if the runner runs in test mode. */
            readonly test?: boolean;
            /** Specifies the snapshot to restore (this will start the runner when the `start`-prop is omitted). */
            readonly snapshot?: ISnapshot<Data>;
            /** Specifies data for instances. */
            readonly data?: (pInstance: Instance) => void;
        });
        /** Retrieves the namespace for the runner. */
        get namespace(): INamespace;
        /** Retrieves the definition. */
        get definition(): Readonly<IDefinition> | undefined;
        /** Sets the definition. */
        set definition(pDefinition: IDefinition | undefined);
        /** Retrieves a reference to the active instance. */
        get instance(): Instance | undefined;
        /** Retrieves the storyline of the running instance. */
        get storyline(): Storyline<T> | undefined;
        /** Retrieves the name of the definition. */
        get name(): string;
        /** Retrieves the mode of operation. */
        get mode(): TModes;
        /** Sets the operation mode. */
        set mode(sMode: TModes);
        /** Retrieves if the preview mode is enabled. */
        get isPreview(): boolean;
        /** Sets the preview mode. */
        set isPreview(bPreview: boolean);
        /** Retrieves the test flag. */
        get isTest(): boolean;
        /** Sets the test flag. */
        set isTest(bTest: boolean);
        /** Retrieves the status of the runner. */
        get status(): TStatus;
        /** Retrieves if the runner is empty. */
        get isEmpty(): boolean;
        /** Retrieves the prologue of the loaded definition. */
        get prologue(): IPrologue | undefined;
        /** Retrieves the epilogue of the loaded definition. */
        get epilogue(): IEpilogueContext;
        /**
         * Invoked when the runner is updated.
         * @param pInstance Reference to the instance.
         * @param sChange Specifies the type of change.
         * @param pStoryline Contains the storyline (if the instance is running).
         * @event
         */
        protected onInstanceChange(
            pInstance: Instance,
            sChange: TChanges,
            pStoryline: Storyline<T> | undefined
        ): void;
        /**
         * Invoked when an instance is started.
         * @param pInstance Reference to the instance.
         * @event
         */
        protected onInstanceStart(pInstance: Instance): void;
        /**
         * Invoked when the collected data of an instance is changed.
         * @param pInstance Reference to the instance.
         * @param pValueChangeEvent Reference to a value change event.
         * @event
         */
        protected onInstanceData(
            pInstance: Instance,
            pValueChangeEvent: IValueChangeEvent
        ): void;
        /**
         * Invoked when an instance is processing.
         * @param pInstance Reference to the instance.
         * @param bProcessing Specifies if the instance is processing.
         * @event
         */
        protected onInstanceProcessing(
            pInstance: Instance,
            bProcessing: boolean
        ): void;
        /**
         * Invoked when an instance is about to finish.
         * @param pInstance Reference to the instance.
         * @param fnFinish A callback function that can be used to control the finish process when you override this event and return `false`.
         * @return Return `true` if the instance finish is allowed or `false` if the instance finish should wait till the callback function is invoked.
         * @event
         */
        protected onInstanceFinishing(
            pInstance: Instance,
            fnFinish: (bFinish: boolean, sReference?: string) => void
        ): boolean;
        /**
         * Invoked when an instance ends. This event is always invoked
         * when an instance is finished, stopped or paused.
         * @param pInstance Reference to the instance.
         * @param sType Specifies why the instance has ended. Can be one of the
         * following values:
         * - `finished`: The instance has finished;
         * - `stopped`: The instance was forced to stop;
         * - `paused`: The instance was paused.
         * @event
         */
        protected onInstanceEnd(
            pInstance: Instance,
            sType: "finished" | "stopped" | "paused"
        ): void;
        /**
         * Invoked when a cluster is entered.
         * @param pContext Reference to the context.
         * @param pCluster Reference to the cluster to render.
         * @param pObserver Reference to the await pointer that signals the
         * runner to validate the cluster and make a step forward. If the
         * mode is set to `progressive` this is done automatically.
         * @event
         */
        onInstanceCluster(
            pContext: Context,
            pCluster: Cluster,
            pObserver: Await
        ): void;
        /**
         * Invoked when a cluster starts or stops evaluating.
         * @param pInstance Reference to the instance.
         * @param pCluster Reference to the cluster that is evaluating.
         * @param bEvaluating Specifies if the cluster is evaluating.
         * @event
         */
        onInstanceClusterEvaluating(
            pInstance: Instance,
            pCluster: Cluster,
            bEvaluating: boolean
        ): void;
        /**
         * Invoked when a cluster is left.
         * @param pInstance Reference to the instance.
         * @param sDirection Specifies the step direction.
         * @param pCluster Reference to the cluster that is left.
         * @event
         */
        onInstanceClusterLeave(
            pInstance: Instance,
            sDirection: "forward" | "backward",
            pCluster: Cluster
        ): void;
        /**
         * Invoked when a branch is left.
         * @param pContext Reference to the context.
         * @param pBranch Reference to the branch that is left.
         * @event
         */
        onInstanceBranchLeave(pContext: Context, pBranch: Branch): void;
        /**
         * Invoked when a node is entered.
         * @param pContext Reference to the context.
         * @param pNode Reference to the node to render.
         * @param pObserver Reference to the await pointer that signals the
         * runner to validate the node. If the mode is set to `progressive` this
         * is done automatically.
         * @event
         */
        onInstanceNode(
            pContext: Context,
            pNode: Node<T>,
            pObserver: Await
        ): void;
        /**
         * Invoked when the progress changes.
         * @param pInstance Reference to the instance.
         * @param nPercentage Contains the progress percentage.
         * @event
         */
        onInstanceProgress(pInstance: Instance, nPercentage: number): void;
        /**
         * Invoked when the instance makes a step.
         * @param pInstance Reference to the instance.
         * @param sDirection Specifies the step direction.
         * @event
         */
        onInstanceStep(
            pInstance: Instance,
            sDirection: "forward" | "backward"
        ): void;
        /**
         * Invoked when the cluster is validated.
         * @param pInstance Reference to the instance.
         * @param sResult Specifies the validation state.
         * @param sType Specifies if the validation type.
         * @event
         */
        onInstanceValidated(
            pInstance: Instance,
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate"
        ): void;
        /**
         * Restores a runner snapshot.
         * @param pSnapshot Specifies the snapshot data.
         * @return Returns a reference to the instance.
         */
        restore(pSnapshot: ISnapshot<Data>): Instance | undefined;
        /**
         * Reloads the runner with the specified definition. During a reload
         * the runner tries to maintain any collected data.
         * @param pDefinition Specifies the definition.
         * @param bOnlyRestartWhenRunning Specifies to only restart the runner
         * if it was running before the reload was requested.
         * @return Returns the new instance or `undefined` if no new instance was started.
         */
        reload(
            pDefinition: IDefinition,
            bOnlyRestartWhenRunning?: boolean
        ): Instance | undefined;
        /**
         * Restarts the runner.
         * @param bPreserveData Specifies if the data of an existing running
         * instance should be preserved (defaults to `false`).
         * @return Returns a reference to the instance.
         */
        restart(bPreserveData?: boolean): Instance;
        /**
         * Rerenders the runner for the specified instance.
         * @param pInstance Reference to the instance.
         */
        rerender(pInstance: Instance): void;
        /**
         * Step forward in the instance.
         * @return Returns `true` if the step succeeded.
         */
        stepForward(): boolean;
        /**
         * Step backward in the active instance.
         * @return Returns `true` if the step succeeded.
         */
        stepBackward(): boolean;
        /**
         * Step to the start of the active instance.
         * @return Returns `true` if the step succeeded.
         */
        stepToStart(): boolean;
        /**
         * Step to the head in the history of the active instance.
         * @return Returns `true` if the step succeeded.
         */
        stepToHead(): boolean;
        /**
         * Finish the active instance when it is finishable.
         * @return Returns a promise that resolves when the instance was finished.
         */
        finish(): Promise<void> | false;
    }
}

declare module "tripetto-runner-foundation/lib/events" {
    /** Dependencies */
    import { RunnerFoundation } from "tripetto-runner-foundation/lib/runner";
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Instance } from "@tripetto/vfsm";
    import { IStoryline } from "tripetto-runner-foundation/lib/storyline/interface";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    import { TChanges } from "tripetto-runner-foundation/lib/changes";
    export type TRunnerFoundationEvents = "OnChange" | "OnRestart" | "OnReload";
    interface IRunnerFoundationEvent<
        H extends THooks,
        T extends NodeBlock,
        Data
    > extends IHookPayload<H> {
        /** Reference to the runner. */
        readonly runner: RunnerFoundation<T, Data>;
    }
    export interface IRunnerFoundationChangeEvent<
        T extends NodeBlock = NodeBlock,
        Data = undefined
    > extends IRunnerFoundationEvent<"OnChange", T, Data> {
        /** Specifies the type of change. */
        readonly change: TChanges;
        /** Reference to the instance. */
        readonly instance: Instance | undefined;
        /** Reference to the storyline. */
        readonly storyline: IStoryline<T>;
    }
    export interface IRunnerFoundationRestartEvent<
        T extends NodeBlock = NodeBlock,
        Data = undefined
    > extends IRunnerFoundationEvent<"OnRestart", T, Data> {
        /** Reference to the instance. */
        readonly instance: Instance;
    }
    export {};
}

declare module "tripetto-runner-foundation/lib/nodes/node" {
    /** Dependencies */
    import { Await, Callback } from "@marblecore/callback";
    import { Cluster } from "tripetto-runner-foundation/lib/clusters/cluster";
    import { RunnerFoundation } from "tripetto-runner-foundation/lib/runner";
    import { ConditionResult } from "tripetto-runner-foundation/lib/decorators/condition";
    import { Context, Transducer, Transducers } from "@tripetto/vfsm";
    import { INode } from "@tripetto/map";
    import { ValidatorResult } from "tripetto-runner-foundation/lib/decorators/validate";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    import { HeadlessBlock } from "tripetto-runner-foundation/lib/nodes/blocks/headless";
    export class Node<
        T extends NodeBlock | HeadlessBlock = NodeBlock
    > extends Transducer<INode> {
        /**
         * Creates a new node instance.
         * @param pNodes Reference to the nodes dispatcher.
         * @param pNode Reference to the node properties.
         */
        constructor(pNodes: Transducers, pNode: INode);
        /** Retrieves the identifier of the node. */
        get id(): string;
        /** Reference to the runner. */
        get runner(): RunnerFoundation;
        /** Reference to the cluster. */
        get cluster(): Cluster;
        /**
         * Retrieves the block instance.
         * @param pContext Reference to the context.
         * @return Returns the block instance or `undefined` if the block was not found.
         */
        block(pContext: Context): T | undefined;
        /**
         * Invoked when the runner wants to verify the condition for the node.
         * This event is invoked just before the node is entered.
         * @param pContext Reference to the context.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous condition callback.
         * @return Returns the result of the condition or the callback reference if
         * the condition should be handled asynchronously.
         * @event
         */
        onEnterCondition(
            pContext: Context,
            pCallback: Callback<boolean>
        ): ConditionResult;
        /**
         * Invoked when the node is entered.
         * @param pContext Reference to the context.
         * @param pAwait Reference to the await pointer.
         * @event
         */
        onEnter(pContext: Context, pAwait: Await): Await | void;
        /**
         * Invoked when the node is left.
         * @param pContext Reference to the context.
         * @event
         */
        onLeave(pContext: Context): void;
        /**
         * Invoked when the transducer is canceled.
         * @param pContext Reference to the context.
         * @event
         */
        onCancel(pContext: Context): void;
        /**
         * Invoked when the node needs to be validated.
         * @param pContext Reference to the context.
         * @param sCurrent Specifies the current node validation state.
         * @param pCallback Reference to the callback pointer. Pass this pointer as
         * return value to enable the asynchronous validation callback.
         * @return Returns the result of the validation or the callback reference if
         * the validation should be handled asynchronously.
         * @event
         */
        onValidate(
            pContext: Context,
            sCurrent: "unknown" | "fail" | "pass",
            pCallback: Callback<boolean>
        ): ValidatorResult;
        /**
         * Invoked when the node is validated.
         * @param pContext Reference to the context.
         * @param sResult Specifies the validation result.
         * @param sType Specifies the validation type.
         * @event
         */
        onValidated(
            pContext: Context,
            sResult: "fail" | "pass",
            sType: "initial" | "revalidate"
        ): void;
        /**
         * Invoked when a side effect is generated.
         * @param pContext Reference to the context.
         * @event
         */
        onSideEffect(pContext: Context): void;
    }
}

declare module "tripetto-runner-foundation/lib/nodes/blocks/block" {
    import { Context, IVariable, ImmutableValue, Value } from "@tripetto/vfsm";
    import { INode, INodeBlock } from "@tripetto/map";
    import { INodeBlockType } from "tripetto-runner-foundation/lib/nodes/blocks/type";
    import { Kinds, Slot } from "@tripetto/slots";
    import { Node } from "tripetto-runner-foundation/lib/nodes/node";
    import { TSerializeTypes } from "@marblecore/serializer";
    export abstract class NodeBlock<Properties extends {} = {}> {
        /** Contains the type name of the block. */
        static readonly identifier: string;
        /** Contains an optional ref. */
        static readonly ref?: {};
        /** Specifies if the block needs an automatic render upon value store. */
        static readonly autoRender: boolean;
        /** Specifies if the block needs automatic validation. */
        static readonly autoValidate: boolean;
        /** Retrieves the block type. */
        readonly type: INodeBlockType<NodeBlock<Properties>>;
        /** Reference to the context. */
        readonly context: Context;
        /**
         * Creates a new node block instance.
         * @param pNode Reference to the node.
         * @param pContext Reference to the context.
         */
        constructor(pNode: Node, pContext: Context);
        /** Retrieves the current view of the runner. */
        get view(): "live" | "test" | "preview";
        /** Retrieves the validation state of the block. */
        get validation(): "unknown" | "pass" | "fail";
        /** Reference to the node props. */
        get node(): Readonly<INode>;
        /** Contains the block props. */
        get props(): Readonly<Properties & INodeBlock>;
        /** Specifies if an automatic render is needed upon value store. */
        get shouldAutoRender(): boolean;
        /** Specifies if automatic validation is applied. */
        get shouldAutoValidate(): boolean;
        /** Retrieves if the validation of the block failed. */
        get isFailed(): boolean;
        /** Retrieves if the validation of the block passed. */
        get isPassed(): boolean;
        /**
         * Retrieves a slot.
         * @param sReference Specifies the slot reference.
         * @param sKind Specifies the slot kind.
         * @return Returns the slot or `undefined` if the slot was not found.
         */
        protected slotOf<S extends Slot = Slot>(
            sReference: string,
            sKind?: Kinds
        ): S | undefined;
        /**
         * Retrieves a mutable value. You can only retrieve mutable values for the
         * slots managed by the block. If you want values from other nodes, use the
         * `immutableValueOf` method.
         * @param pReference Specifies the slot reference.
         * @param sKind Specifies the slot kind.
         * @param pOptions Specifies optional options.
         * @return Returns the mutable value or `undefined` if the value was not found.
         */
        protected valueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(
            pReference: Slot | string,
            sKind?: Kinds,
            pOptions?: {
                /** Specifies a prefill value. */
                readonly prefill?: {
                    readonly value: TSerializeTypes;
                    readonly reference?: string;
                };
                /** Specifies a modifier function that is invoked when the data is about to change. */
                readonly modifier?: (
                    pValue: Value<T, S>
                ) =>
                    | {
                          readonly value: TSerializeTypes;
                          readonly reference?: string;
                      }
                    | undefined;
                /** Invoked when a value is changed. */
                readonly onChange?: (pValue: Value<T, S>) => void;
                /** Invoked when a value is used within another context. */
                readonly onContext?: (
                    pValue: Value<T, S>,
                    pContext: Context
                ) => void;
            }
        ): Value<T, S> | undefined;
        /**
         * Retrieves an immutable value. You can retrieve immutable values for any node.
         * @param pReference Specifies the slot instance or identifier.
         * @return Returns the immutable value or `undefined` if the value was not found.
         */
        protected immutableValueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(pReference: Slot | string): ImmutableValue<T, S> | undefined;
        /**
         * Retrieves a variable.
         * @param pReference Specifies the slot reference, value or variable identifier.
         * @return Returns the variable or `undefined` if the variable was not found.
         */
        protected variableFor(
            pReference:
                | Slot
                | {
                      slot: Slot;
                  }
                | string
        ): IVariable | undefined;
        /**
         * Parses the supplied string, replacing all variables with their current values.
         * @param sString Specifies the string to parse.
         * @param sPlaceholder Specifies a string for empty variables (disabled by default).
         * @param bLineBreaks Specifies if line breaks are supported (disabled by default).
         * @return Returns the parsed string.
         */
        protected parseVariables(
            sString: string,
            sPlaceholder?: string,
            bLineBreaks?: boolean
        ): string;
        /**
         * Retrieves a unique key for the block within the current context for the
         * specified label (the key is prefixed with an underscore). Use this function
         * if you need stable unique keys (for example for use as element identifiers).
         * @param sLabel Optional label for the key (if omitted the default key will
         * be returned).
         * @return Returns the key.
         */
        key(sLabel?: string): string;
        /** Invokes a block rerendering. */
        rerender(): void;
        /** Locks the data values of a block. */
        lock(): void;
        /** Unlocks the data values of a block. */
        unlock(): void;
        /** Clears the data values of a block. */
        clear(): void;
    }
}

declare module "tripetto-runner-foundation/lib/nodes/blocks/headless" {
    /** Dependencies */
    import { Await } from "@marblecore/callback";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    import { Value } from "@tripetto/vfsm";
    import { Slot } from "@tripetto/slots";
    import { TSerializeTypes } from "@marblecore/serializer";
    export abstract class HeadlessBlock<
        Properties extends {} = {}
    > extends NodeBlock<Properties> {
        /**
         * Retrieves the mutable value for the supplied slot instance or identifier.
         * @param pReference Specifies the slot instance or identifier.
         * @return Returns the value or `undefined` if no value was found.
         */
        protected mutableValueOf<
            T extends TSerializeTypes,
            S extends Slot<T> = Slot<T>
        >(pReference: Slot | string): Value<T, S> | undefined;
        /**
         * Implements the operation for the block.
         * @param pDone Reference to an await pointer that can be used for asynchronous operations.
         */
        abstract do(pDone?: Await): Await | void;
    }
}

declare module "tripetto-runner-foundation/lib/nodes/blocks/blocks" {
    /** Dependencies */
    import { Context } from "@tripetto/vfsm";
    import { Node } from "tripetto-runner-foundation/lib/nodes/node";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    import { HeadlessBlock } from "tripetto-runner-foundation/lib/nodes/blocks/headless";
    import { TNodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/type";
    export class NodeBlocksNamespace {
        /** Creates a new namespace instance. */
        constructor(sIdentifier: string);
        /** Retrieves the identifier for the namespace. */
        get identifier(): string;
        /**
         * Registers the supplied block.
         * @param pBlock Specifies the block.
         * @param sIdentifier Specifies the block type identifier.
         * @param pAliases Specifies type aliases for the block.
         * @param pRef Specifies an optional ref to something.
         * @param bAutoRender Specifies if the block needs to be rerendered upon
         * value store (disabled by default).
         * @param bAutoValidate Specifies if the block checks if all required
         * slots contain a value (enabled by default).
         */
        register(
            pBlock: TNodeBlock,
            sIdentifier: string,
            pAliases: string[],
            pRef?: {},
            bAutoRender?: boolean,
            bAutoValidate?: boolean
        ): void;
        /**
         * Creates a new instance of a node block for the supplied node.
         * @param pNode Reference to the node.
         * @param pContext Reference to the context.
         * @return Returns a node block instance.
         */
        create<T extends NodeBlock | HeadlessBlock>(
            pNode: Node<T>,
            pContext: Context
        ): T | undefined;
    }
}

declare module "tripetto-runner-foundation/lib/nodes/blocks/type" {
    /** Dependencies */
    import { Context } from "@tripetto/vfsm";
    import { Node } from "tripetto-runner-foundation/lib/nodes/node";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    export interface INodeBlockType<T extends NodeBlock> {
        /** Contains the block type identifier. */
        readonly identifier: string;
        /** Optional reference to something. */
        readonly ref?: {};
        /** Specifies if the block needs an automatic render upon value store. */
        readonly autoRender?: boolean;
        /** Specifies if the block needs automatic validation. */
        readonly autoValidate?: boolean;
        /**
         * Creates a new block instance.
         * @param pNode Reference to the node.
         * @param pContext Reference to the context.
         */
        new (pNode: Node, pContext: Context): T;
    }
    export type TNodeBlock = INodeBlockType<NodeBlock>;
}

declare module "tripetto-runner-foundation/lib/storyline/observable" {
    /** Dependencies */
    import { Await } from "@marblecore/callback";
    import { Context } from "@tripetto/vfsm";
    import { INode } from "@tripetto/map";
    import { Node } from "tripetto-runner-foundation/lib/nodes/node";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    export interface IObservableNode<T extends NodeBlock> {
        /** Contains a unique identifier for the node. */
        readonly id: string;
        /** Contains a unique key for the node within the current context (the key is prefixed with an underscore). */
        readonly key: string;
        /** Reference to the node. */
        readonly node: Node;
        /** Reference to the node properties. */
        readonly props: INode;
        /** Reference to the node block. */
        readonly block: T | undefined;
        /** Reference to the context. */
        readonly context: Context;
        /** Specifies if the node collects data. */
        readonly collectsData: boolean;
        /** Specifies if the node has collected data. */
        readonly hasDataCollected: boolean;
        /**
         * Contains the enumerator for the node. Enumerators are only granted to nodes
         * that have a block that collects data.
         */
        readonly enumerator: number | undefined;
        /** Contains the validation state of the node. */
        readonly validation: "unknown" | "pass" | "fail";
        /** Retrieves if the validation of the node failed. */
        readonly isFailed: boolean;
        /** Retrieves if the validation of the node passed. */
        readonly isPassed: boolean;
        /** Reference to the node observer. */
        readonly observer: Await | undefined;
    }
}

declare module "tripetto-runner-foundation/lib/changes" {
    export type TChanges =
        | "started"
        | "render"
        | "evaluating"
        | "validated"
        | "forward"
        | "backward"
        | "progress"
        | "finishing"
        | "finished"
        | "stopped"
        | "paused"
        | "mode"
        | "preview";
}

declare module "tripetto-runner-foundation/lib/modes" {
    export type TModes = "paginated" | "continuous" | "progressive";
}

declare module "tripetto-runner-foundation/lib/status" {
    export type TStatus =
        | "idle"
        | "empty"
        | "preview"
        | "running"
        | "pausing"
        | "paused"
        | "stopped"
        | "finishing"
        | "finished";
}

declare module "tripetto-runner-foundation/lib/markdown" {
    /** Dependencies */
    import {
        MarkdownFeatures,
        MarkdownParser,
        MarkdownTypes,
    } from "@marblecore/markdown";
    import { Context, IVariable } from "@tripetto/vfsm";
    /**
     * Runs the string through the markdown parser.
     * @param sMarkdown Specifies the markdown string to parse.
     * @param pContext Specifies the context.
     * @param pOptions Specifies the markdown options.
     * @return Returns a markdown parser instance.
     */
    export function markdownify(
        sMarkdown: string,
        pContext: Context | undefined,
        pOptions?: {
            /** Specifies the supported markdown features (defaults to `MarkdownFeatures.Formatting | MarkdownFeatures.Hyperlinks`). */
            features?: MarkdownFeatures;
            /** Specifies the placeholder for empty variables. */
            placeholder?: string;
            /** Specifies if line breaks are supported. */
            lineBreaks?: boolean;
        }
    ): MarkdownParser<IVariable>;
    /**
     * Parses a markdown string to a plain text string.
     * @param sMarkdown Specifies the markdown string to parse.
     * @param pContext Specifies the context.
     * @param sPlaceholder Specifies a string for empty variables (defaults to `___`).
     * @param bLineBreaks Specifies if line breaks are supported (disabled by default).
     * @param nFeatures Specifies the features to parse (defaults to `MarkdownFeatures.None`).
     * @return Returns a plain text string.
     */
    export function markdownifyToString(
        sMarkdown: string,
        pContext: Context | undefined,
        sPlaceholder?: string,
        bLineBreaks?: boolean,
        nFeatures?: MarkdownFeatures
    ): string;
    /**
     * Parses a markdown string to a valid URL.
     * @param sMarkdown Specifies the markdown string to parse.
     * @param pContext Specifies the context.
     * @param sPlaceholder Optional placeholder to use (disabled by default).
     * @return Returns the URL.
     */
    export function markdownifyToURL(
        sMarkdown: string,
        pContext: Context | undefined,
        sPlaceholder?: string
    ): string;
    /**
     * Reduces or maps a markdown string to a certain structure.
     * @param sMarkdown Specifies the markdown string to parse.
     * @param pContext Specifies the context.
     * @param pOptions Specifies the markdown options.
     * @return Returns the parsed structure.
     */
    export function markdownifyTo<T>(
        sMarkdown: string,
        pContext: Context | undefined,
        pOptions: (
            | {
                  reduce: (
                      sType: MarkdownTypes | undefined,
                      pContent: string | T[],
                      pValue: string | IVariable | undefined
                  ) => T;
              }
            | {
                  map: (
                      sType: MarkdownTypes | undefined,
                      sContent: string,
                      pValue: string | IVariable | undefined,
                      pParent: T | undefined,
                      sKind: "root" | "text" | "node"
                  ) => T;
              }
        ) & {
            /** Specifies the supported markdown features (defaults to `MarkdownFeatures.Formatting | MarkdownFeatures.Hyperlinks`). */
            features?: MarkdownFeatures;
            /** Specifies the placeholder for empty variables. */
            placeholder?: string;
            /** Specifies if line breaks are supported (default is `false`). */
            lineBreaks?: boolean;
        }
    ): T;
    /**
     * Converts markdown to plain text, replacing mentions with a placeholder string and removing formatting and hyperlink markdown.
     * @param sMarkdown Specifies the markdown.
     * @return Returns a plain text string.
     */
    export function markdownifyToPlainText(sMarkdown: string): string;
}

declare module "tripetto-runner-foundation/lib/variables" {
    import { Context, ImmutableValue } from "@tripetto/vfsm";
    export function localizeVariable(
        pValue: ImmutableValue,
        pContext: Context
    ): string | undefined;
    /**
     * Verifies if the supplied input is a possible variable.
     * @param sInput Specifies the input to verify.
     * @return Returns `true` if the input could be a variable.
     */
    export function isVariable(sInput: string | undefined): sInput is string;
}

declare module "tripetto-runner-foundation/lib/storyline/storyline" {
    /** Dependencies */
    import { Cluster } from "tripetto-runner-foundation/lib/clusters/cluster";
    import { IPrologue } from "@tripetto/map";
    import { Instance } from "@tripetto/vfsm";
    import { IObservableNode } from "tripetto-runner-foundation/lib/storyline/observable";
    import { IPage } from "tripetto-runner-foundation/lib/storyline/page";
    import { IStoryline } from "tripetto-runner-foundation/lib/storyline/interface";
    import { IStorylineOrchestrator } from "tripetto-runner-foundation/lib/storyline/orchestrator";
    import { Moment } from "tripetto-runner-foundation/lib/storyline/moment";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    import { TModes } from "tripetto-runner-foundation/lib/modes";
    import { IEpilogueContext } from "tripetto-runner-foundation/lib/epilogue";
    export class Storyline<T extends NodeBlock = NodeBlock> {
        /**
         * Creates a new storyline.
         * @param pOrchestrator Reference to the storyline orchestrator.
         */
        constructor(pOrchestrator: IStorylineOrchestrator);
        /** Retrieves the storyline checksum. */
        get checksum(): string | undefined;
        /** Retrieves an immutable representation of the storyline. */
        get immutable(): IStoryline<T>;
        /** Retrieves the instance of the storyline. */
        get instance(): Instance;
        /** Retrieves the collection of moments. */
        get moments(): Moment<T>[];
        /** Retrieves the collection of pages (only available in paginated mode). */
        get pages(): IPage[];
        /** Retrieves the active page. */
        get activePage(): IPage | undefined;
        /** Retrieves the present moment. */
        get presentMoment(): Moment<T> | undefined;
        /** Contains the cluster of the present moment. */
        get presentCluster(): Cluster | undefined;
        /** Contains the observable nodes of the present moment. */
        get presentNodes(): IObservableNode<T>[];
        /** Contains the observable nodes of the active moments. */
        get activeNodes(): IObservableNode<T>[];
        /** Retrieves if the storyline is evaluating. */
        get isEvaluating(): boolean;
        /** Retrieves if the storyline has changed since the last call of this prop. */
        get isChanged(): boolean;
        /** Retrieves if the storyline is empty (has no observable nodes). */
        get isEmpty(): boolean;
        /** Retrieves if the present moment is at the begin of the storyline. */
        get isAtStart(): boolean;
        /** Retrieves if the present moment is at the head of the storyline. */
        get isAtHead(): boolean;
        /** Retrieves if the present moment is at the finish of the storyline. */
        get isAtFinish(): boolean;
        /** Retrieves if the storyline is finishable using the `finish()` method. */
        get isFinishable(): boolean;
        /** Retrieves if the storyline is finishing. */
        get isFinishing(): boolean;
        /** Retrieves if the storyline is finished. */
        get isFinished(): boolean;
        /** Retrieves if the storyline is pausing. */
        get isPausing(): boolean;
        /** Retrieves if the storyline is paused. */
        get isPaused(): boolean;
        /** Retrieves if the storyline has any data collected. */
        get hasDataCollected(): boolean;
        /** Retrieves if the validation of the storyline failed. */
        get isFailed(): boolean;
        /** Retrieves if the validation of the storyline passed. */
        get isPassed(): boolean;
        /** Retieves the mode of operation. */
        get mode(): TModes;
        /** Retrieves the most recent step direction of the storyline. */
        get direction(): "forward" | "backward";
        /** Total number of observable nodes in the storyline. */
        get count(): number;
        /** Total number of observable nodes that have an enumerator value. */
        get enumerators(): number;
        /** Retrieves the progress percentage of the storyline. */
        get percentage(): number;
        /** Retrieves the prologue of the storyline. */
        get prologue(): IPrologue | undefined;
        /** Retrieves the epilogue of the storyline. */
        get epilogue(): IEpilogueContext;
        /** Invalidates the current storyline. */
        invalidate(): void;
        /** Resets the storyline and clears the moments cache. */
        reset(): void;
        /**
         * Maps moments to a certain type.
         * @param fnMoment Specifies the function to invoke for each moment.
         */
        map<R>(fnMoment: (pMoment: Moment<T>, nIndex: number) => R): R[];
        /**
         * Step forward in the storyline.
         * @return Returns `true` if the step succeeded.
         */
        stepForward(): boolean;
        /**
         * Step backward in the storyline.
         * @return Returns `true` if the step succeeded.
         */
        stepBackward(): boolean;
        /**
         * Step to the start of the storyline.
         * @return Returns `true` if the step succeeded.
         */
        stepToStart(): boolean;
        /**
         * Step to the head of the storyline.
         * @return Returns `true` if the step succeeded.
         */
        stepToHead(): boolean;
        /**
         * Steps to a page.
         * @param nPage Specifies the page to step to.
         * @return Returns `true` if the step succeeded.
         */
        stepToPage(nPage: number): boolean;
        /**
         * Finish the storyline when it is at the finish.
         * @return Returns a promise that resolves when the instance was finished.
         */
        finish(): Promise<void>;
        /**
         * Find the node that belongs to the supplied key.
         * @param sKey Specifies the key to use.
         * @return Returns the node of `undefined` if the node is not found.
         */
        getNodeByKey(sKey: string): IObservableNode<T> | undefined;
        /**
         * Finds the first key for the supplied node identifier.
         * @param sId Specifies the node id.
         * @return Returns the key of the node or `undefined` if the node is not found.
         */
        getKeyByNodeId(sId: string): string | undefined;
    }
}

declare module "tripetto-runner-foundation/lib/storyline/interface" {
    /** Dependencies */
    import { Cluster } from "tripetto-runner-foundation/lib/clusters/cluster";
    import { IObservableNode } from "tripetto-runner-foundation/lib/storyline/observable";
    import { IPage } from "tripetto-runner-foundation/lib/storyline/page";
    import { Moment } from "tripetto-runner-foundation/lib/storyline/moment";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    import { Storyline } from "tripetto-runner-foundation/lib/storyline/storyline";
    import { TModes } from "tripetto-runner-foundation/lib/modes";
    export interface IStory<T extends NodeBlock> {
        /** Storyline instance. */
        readonly storyline: Storyline<T>;
        /** Storyline direction. */
        direction?: "forward" | "backward";
        /** Storyline checksum. */
        checksum?: string;
    }
    export interface IStoryline<T extends NodeBlock = NodeBlock> {
        /** Retrieves the collection of moments. */
        readonly moments: Moment<T>[];
        /** Retrieves the collection of pages (only available in paginated mode). */
        readonly pages: IPage[];
        /** Retrieves the present moment. */
        readonly presentMoment: Moment<T> | undefined;
        /** Contains the cluster of the present moment. */
        readonly presentCluster: Cluster | undefined;
        /** Contains the observable nodes of the present moment. */
        readonly presentNodes: IObservableNode<T>[];
        /** Contains the observable nodes of the active moments. */
        readonly activeNodes: IObservableNode<T>[];
        /** Retrieves if the storyline is evaluating. */
        readonly isEvaluating: boolean;
        /** Retrieves if the storyline is empty (has no observable nodes). */
        readonly isEmpty: boolean;
        /** Retrieves if the present moment is at the begin of the storyline. */
        readonly isAtStart: boolean;
        /** Retrieves if the present moment is at the head of the storyline. */
        readonly isAtHead: boolean;
        /** Retrieves if the present moment is at the finish of the storyline. */
        readonly isAtFinish: boolean;
        /** Retrieves if the storyline is finishable using the `finish()` method. */
        readonly isFinishable: boolean;
        /** Retrieves if the storyline is finishing. */
        readonly isFinishing: boolean;
        /** Retrieves if the storyline is finished. */
        readonly isFinished: boolean;
        /** Retrieves if the storyline is pausing. */
        readonly isPausing: boolean;
        /** Retrieves if the storyline is paused. */
        readonly isPaused: boolean;
        /** Retrieves if the storyline has any data collected. */
        readonly hasDataCollected: boolean;
        /** Retrieves if the validation of the storyline failed. */
        readonly isFailed: boolean;
        /** Retrieves if the validation of the storyline passed. */
        readonly isPassed: boolean;
        /** Retieves the mode of operation. */
        readonly mode: TModes;
        /** Retrieves the most recent step direction of the storyline. */
        readonly direction: "forward" | "backward";
        /** Total number of observable nodes in the storyline. */
        readonly count: number;
        /** Total number of observable nodes that have an enumerator value. */
        readonly enumerators: number;
        /** Retrieves the progress percentage of the storyline. */
        readonly percentage: number;
        /** Maps moments to a certain type. */
        readonly map: <R>(
            fnMoment: (pMoment: Moment<T>, nIndex: number) => R
        ) => R[];
        /** Step forward in the storyline. */
        readonly stepForward: () => boolean;
        /** Step backward in the storyline. */
        readonly stepBackward: () => boolean;
        /** Step to the start of the storyline. */
        readonly stepToStart: () => boolean;
        /** Step to the head of the storyline. */
        readonly stepToHead: () => boolean;
        /** Steps to a page. */
        readonly stepToPage: (nPage: number) => boolean;
        /** Finish the storyline when it is at the finish. */
        readonly finish: () => Promise<void> | false;
    }
}

declare module "tripetto-runner-foundation/lib/storyline/moment" {
    import { Cluster } from "tripetto-runner-foundation/lib/clusters/cluster";
    import { IMoment } from "@tripetto/vfsm";
    import { IObservableNode } from "tripetto-runner-foundation/lib/storyline/observable";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    export class Moment<T extends NodeBlock = NodeBlock> {
        /**
         * Creates a new moment.
         * @param pMoment Reference to the moment.
         * @param nEnumerator Specifies the enumerator value.
         */
        constructor(pMoment: IMoment, nEnumerator: number);
        /** Reference to the cluster. */
        get cluster(): Cluster;
        /** Collection of nodes. */
        get nodes(): IObservableNode<T>[];
        /** Retrieves the tense of the moment. */
        get tense(): "past" | "present" | "future";
        /** Specifies if the moment is in the past. */
        get isPast(): boolean;
        /** Specifies if the moment is the present moment. */
        get isPresent(): boolean;
        /** Specifies if the moment is in the future. */
        get isFuture(): boolean;
        /** Retrieves if the moment is evaluating. */
        get isEvaluating(): boolean;
        /** Retrieves if the validation of the cluster failed. */
        get isFailed(): boolean;
        /** Retrieves if the validation of the cluster passed. */
        get isPassed(): boolean;
        /** Total number of nodes in the moment. */
        get count(): number;
        /** Total number of nodes that have an enumerator value. */
        get enumerators(): number;
        /** Restores the moment. */
        restore(): void;
    }
}

declare module "tripetto-runner-foundation/lib/storyline/page" {
    export interface IPage {
        /** Contains the page number. */
        readonly number: number;
        /** Contains if the page is active. */
        readonly active: boolean;
        /** Activates the page. */
        readonly activate: () => void;
    }
}

declare module "tripetto-runner-foundation/lib/data/import" {
    /** Dependencies */
    import { Context, IDataImport, Instance } from "@tripetto/vfsm";
    import { TSerializeTypes } from "@marblecore/serializer";
    export namespace Import {
        interface IDataValues {
            /** Data slot values. */
            readonly values: IDataImport;
        }
        /** Describes the values object. */
        interface IValues {
            readonly [slot: string]: IDataValues | undefined;
        }
        /** Describes an importable field value. */
        interface IFieldValue {
            /** Contains the data value. */
            readonly value: TSerializeTypes;
            /** Contains the data reference. */
            readonly reference?: string;
            /** Contains the UTC set time of the data. */
            readonly time?: number;
        }
        /** Describes an importable field by its key. */
        interface IFieldByKey extends IFieldValue {
            /** Key of the data field. */
            readonly key: string;
        }
        /** Describes an importable field by name. */
        interface IFieldByName extends IFieldValue {
            /** Contains the name. */
            readonly name: string;
        }
        /**
         * Import data values to the instance.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pValues Specifies the values to import.
         * @param pSelection Specifies the identifiers of the slots that should be imported.
         * @return Returns `true` if the import succeeded.
         */
        function values(
            pInstanceOrContext: Instance | Context,
            pValues: IValues,
            pSelection?: string[]
        ): boolean;
        /**
         * Import data fields to the instance.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pFields Contains the fields to import. You can reference a field
         * by its key or field name.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @param pSelection Specifies the identifiers of the slots that should be imported.
         * @return Returns `true` if the import succeeded.
         */
        function fields(
            pInstanceOrContext: Instance | Context,
            pFields: (IFieldByKey | IFieldByName)[],
            pSelection?: string[],
            sSeparator?: string
        ): boolean;
        /**
         * Imports CSV data. The number of comma-separated fields and their order
         * should exactly match the number of fields and their order in the instance.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param sRecord Specifies the CSV record to import.
         * @param pSelection Specifies the identifiers of the slots that should be imported.
         * @return Returns `true` if the import succeeded.
         */
        function CSV(
            pInstanceOrContext: Instance | Context,
            sRecord: string,
            pSelection?: string[]
        ): boolean;
    }
}

declare module "tripetto-runner-foundation/lib/data/export" {
    /** Dependencies */
    import { Context, IDataValues, Instance } from "@tripetto/vfsm";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { INode } from "@tripetto/map";
    export namespace Export {
        /** Describes the values object. */
        interface IValues {
            readonly [slot: string]:
                | {
                      /** Unique fingerprint of the definition. */
                      readonly fingerprint: string;
                      /** Node */
                      readonly node: {
                          /** Node identifier. */
                          readonly id: string;
                          /** Friendly name of node. */
                          readonly name: string;
                      };
                      /** Block */
                      readonly block: {
                          /** Contains the block type identifier. */
                          readonly type: string;
                          /** Contains the block version. */
                          readonly version: string;
                      };
                      /** Slot. */
                      readonly slot: {
                          /** Slot identifier. */
                          readonly id: string;
                          /** Reference of slot. */
                          readonly reference: string;
                          /** Alias or name of slot. */
                          readonly name: string;
                          /** Slot label. */
                          readonly label: string;
                      };
                      /** Data slot values. */
                      readonly values: IDataValues;
                  }
                | undefined;
        }
        interface IExportableFieldNode {
            /** Contains the node identifier. */
            readonly id: string;
            /** Contains the key of the node within the context. */
            readonly key: string;
            /** Contains the context for the node. */
            readonly context: string;
            /** Retrieve a reference to the node props. */
            readonly props: () => INode;
        }
        /** Describes an exportable field. */
        interface IExportableField {
            /** A unique key for the data field. */
            readonly key: string;
            /** Contains the name. */
            readonly name: string;
            /** Contains the block type identifier. */
            readonly type: string;
            /** Contains the block version. */
            readonly version: string;
            /** Contains the node. */
            readonly node: IExportableFieldNode;
            /** Contains the slot reference. */
            readonly slot: string;
            /** Contains the slot data type. */
            readonly datatype: string;
            /** Contains the data as a string. */
            readonly string: string;
            /** Contains the data value. */
            readonly value: TSerializeTypes;
            /** Contains if the data was modified. */
            readonly modified: boolean;
            /** Contains the data reference. */
            readonly reference?: string;
            /** Contains the UTC set time of the data. */
            readonly time?: number;
        }
        /** Describes the exportable fields. */
        interface IExportables {
            /** Unique fingerprint of the definition. */
            readonly fingerprint: string;
            /** Contains the stencil for the exportable data. */
            readonly stencil: string;
            /** Contains the fields. */
            readonly fields: IExportableField[];
        }
        /** Describes a CSV export. */
        interface ICSV {
            /** Unique fingerprint of the definition. */
            readonly fingerprint: string;
            /** Contains the stencil for the exportable data. */
            readonly stencil: string;
            /** Contains the CSV fields. */
            readonly fields: string;
            /** Contains the CSV record. */
            readonly record: string;
        }
        /** Decribes the data of an actionable nide. */
        interface IActionableData {
            /** A unique key for the data field. */
            readonly key: string;
            /** Contains the name. */
            readonly name: string;
            /** Contains the slot reference. */
            readonly slot: string;
            /** Contains the slot data type. */
            readonly datatype: string;
            /** Contains the data as a string. */
            readonly string: string;
            /** Contains the data value. */
            readonly value: TSerializeTypes;
            /** Contains if the data was modified. */
            readonly modified: boolean;
            /** Contains the data reference. */
            readonly reference?: string;
            /** Contains the UTC set time of the data. */
            readonly time?: number;
        }
        /** Describes a actionable node. */
        interface IActionableNode {
            /** Contains the node key. */
            readonly key: string;
            /** Contains the block type identifier. */
            readonly type: string;
            /** Contains the block version. */
            readonly version: string;
            /** Contains the node. */
            readonly node: IExportableFieldNode;
            /** Contains the data. */
            readonly data: IActionableData[];
        }
        /** Describes all the actionable nodes. */
        interface IActionables {
            /** Unique fingerprint of the definition. */
            readonly fingerprint: string;
            /** Contains the stencil for the actionable data. */
            readonly stencil: string;
            /** Contains the actionable nodes. */
            readonly nodes: IActionableNode[];
        }
        /**
         * Retrieves a collection of all valid exportable values. The values are stored
         * per slot. The key of each value object is the identifier of the slot. The keys
         * for the contextual values are composed using the condition hashes of the
         * context. If the context is global the key of that value is `*`. If the
         * context array contains a single condition, the key is the computed
         * hash of the stringified condition data (where the `id` property is set to
         * an empty `string`). If the context array contains 2 or more conditions, the
         * computed hashes of these conditions are used to calculate a new hash. The
         * computed condition hashes are concatenated in chronological order and then a
         * `SHA2_256` hash is generated for this concatenated string. This hash is
         * used as key.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @return Returns the values.
         */
        function values(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[]
        ): IValues;
        /**
         * Retrieves an array with all exportable fields. This includes all slots, even
         * the ones that don't contain data. So this array always has the
         * same number of items regardless of the actual collected slots. If you
         * just want the slots with data, use `exportablesWithData`.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns the fields.
         */
        function exportables(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): IExportables;
        /**
         * Retrieves an array with only the exportable fields that contain data.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns the fields.
         */
        function exportablesWithData(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): IExportables;
        /**
         * Retrieves a diff array with only the exportable fields that have changed.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns the fields.
         */
        function exportablesDiff(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): IExportables;
        /**
         * Retrieves an array with all exportable fields. This includes all slots, even
         * the ones that don't contain data. So this array always has the
         * same number of items regardless of the actual collected slots. If you
         * just want the slots with data, use `exportablesWithData`.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns the fields.
         * @deprecated
         */
        function fields(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): IExportables;
        /**
         * Export the exportable fields as name-value pairs.
         * @param reference Reference to the instance, context or an exportables object.
         * @returns Returns an object with name-value pairs.
         */
        function NVPs(
            reference: Instance | Context | IExportables
        ): {
            readonly [name: string]: string | boolean | number | undefined;
        };
        /**
         * Export the exportable fields as name-value pairs (where the value is a string).
         * @param reference Reference to the instance, context or an exportables object.
         * @param modifier Specifies the values should be casted to strings.
         * @returns Returns an object with name-value pairs (where the value is a string).
         */
        function NVPs(
            reference: Instance | Context | IExportables,
            modifier: "strings"
        ): {
            readonly [name: string]: string;
        };
        /**
         * Export the exportable fields as name-value pairs.
         * @param reference Reference to the instance, context or an exportables object.
         * @param modifier Invoked for each field so you can modify the returned value for each field.
         * @returns Returns an object with name-value pairs.
         */
        function NVPs(
            reference: Instance | Context | IExportables,
            modifier: (
                field: IExportableField
            ) => string | boolean | number | undefined
        ): {
            readonly [name: string]: string | boolean | number | undefined;
        };
        /**
         * Retrieves the data in CSV format.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns a CSV object with the fields and the record.
         */
        function CSV(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): ICSV;
        /**
         * Retrieves the exportable fields that contain data in CSV format.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns a CSV object with the fields and the record.
         */
        function CSVWithData(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): ICSV;
        /**
         * Retrieves the diff data in CSV format. Only fields that have changed are
         * included.
         * @param pInstanceOrContext Reference to the instance or context.
         * @param pSelection Specifies the identifiers of the slots that should be exported.
         * @param sSeparator Specifies the string that is used to separate multiple
         * field names (defaults to ` / `).
         * @returns Returns a CSV object with the fields and the record.
         */
        function CSVDiff(
            pInstanceOrContext: Instance | Context,
            pSelection?: string[],
            sSeparator?: string
        ): ICSV;
        /**
         * Retrieves all actionable fields.
         * @param pInstanceOrContext Reference to the instance or context.
         */
        function actionables(
            pInstanceOrContext: Instance | Context
        ): IActionables | undefined;
    }
}

declare module "tripetto-runner-foundation/lib/epilogue" {
    import { IEpilogue } from "@tripetto/map";
    import { Context } from "@tripetto/vfsm";
    export interface IEpilogueContext extends IEpilogue {
        readonly branch?: string;
        readonly context?: Context;
        readonly getRedirect?: () => string | undefined;
    }
}

declare module "tripetto-runner-foundation/lib/namespaces" {
    import { NodeBlocksNamespace } from "tripetto-runner-foundation/lib/nodes/blocks/blocks";
    import { ConditionBlocksNamespace } from "tripetto-runner-foundation/lib/conditions/blocks/blocks";
    export interface INamespace {
        readonly identifier: string;
        readonly nodeBlocks: NodeBlocksNamespace;
        readonly conditionBlocks: ConditionBlocksNamespace;
    }
    export class Namespaces {
        /**
         * Retrieves the namespace for the specified identifier (or the default
         * namespace if the identifier is undefined).
         */
        static get(sIdentifier?: string): INamespace;
        /** Retrieves if the supplied namespace is available. */
        static isAvailable(sIdentifier: string): boolean;
        /**
         * Mounts (activates) a namespace context.
         * @param sIdentifier Specifies the namespace identifier.
         */
        static mount(sIdentifier: string): void;
        /** Unmounts (deactivates) the active namespace context. */
        static unmount(): void;
        /**
         * Loads a namespace.
         * @param sUMD Specifies the UMD code that needs to be loaded.
         * @return Returns `true` if the loading was succesfull.
         */
        static load(sUMD: string): boolean;
        /**
         * Loads a namespace asynchronously.
         * @param fnLoad Load function with a done callback as argument that receives the UMD code when the loading is done.
         * @param fnCallback Invoked when the loading is done.
         */
        static loadAsync(
            fnLoad: (
                fnDone: (sUMD: string) => void,
                fnCancel: () => void
            ) => void,
            fnCallback?: (bSucceeded: boolean) => void
        ): void;
        /** Unloads the specified namespace. */
        static unload(sIdentifier: string): boolean;
    }
    /**
     * Mounts (activates) a namespace context.
     * @param sIdentifier Specifies the namespace identifier.
     */
    export function mountNamespace(sIdentifier: string): void;
    /** Unmounts (deactivates) the active namespace context. */
    export function unmountNamespace(): void;
}

declare module "tripetto-runner-foundation/lib/decorators/block" {
    import { IConditionBlockDecorator } from "tripetto-runner-foundation/lib/conditions/blocks/decorator";
    import {
        IHeadlessBlockDecorator,
        INodeBlockDecorator,
    } from "tripetto-runner-foundation/lib/nodes/blocks/decorator";
    import { TConditionBlock } from "tripetto-runner-foundation/lib/conditions/blocks/type";
    import { TNodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/type";
    /**
     * Defines a node block.
     * @param pProperties Specifies the properties for the block.
     */
    export function tripetto(
        pProperties: INodeBlockDecorator | IHeadlessBlockDecorator
    ): (pBlock: TNodeBlock) => void;
    /**
     * Defines a condition block.
     * @param pProperties Specifies the properties for the block.
     */
    export function tripetto(
        pProperties: IConditionBlockDecorator
    ): (pBlock: TConditionBlock) => void;
}

declare module "tripetto-runner-foundation/lib/decorators/condition" {
    /** Dependencies */
    import { Callback } from "@marblecore/callback";
    import { ConditionBlock } from "tripetto-runner-foundation/lib/conditions/blocks/block";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    export interface IConditionHandler {
        /**
         * Reference to the callback pointer. Pass this pointer as return value of
         * your condition function to enable asynchronous condition callback.
         */
        callback: Callback<boolean>;
    }
    export type ConditionResult = boolean | Callback<boolean>;
    /**
     * Marks the condition function that is invoked when the block is evaluated.
     * The function receives a `IConditionHandler` object.
     * The function should return a value of `ConditionResult`.
     */
    export function condition<T extends NodeBlock | ConditionBlock>(
        pTarget: T,
        sFunction: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "tripetto-runner-foundation/lib/decorators/validate" {
    /** Dependencies */
    import { Callback } from "@marblecore/callback";
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    export interface IValidatorHandler {
        /**
         * Reference to the callback pointer. Pass this pointer as return value to
         * enable the asynchronous validation callback.
         */
        callback: Callback<boolean>;
        /** Contains the current validation state. */
        current: "unknown" | "fail" | "pass";
    }
    export type ValidatorResult = boolean | Callback<boolean>;
    /**
     * Marks the validation function that is invoked when the block is validated.
     * The function receives a `IValidatorHandler` object.
     * The function should return a value of type `ValidatorResult`.
     */
    export function validator<T extends NodeBlock>(
        pTarget: T,
        sFunction: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "tripetto-runner-foundation/lib/decorators/destroy" {
    import { NodeBlock } from "tripetto-runner-foundation/lib/nodes/blocks/block";
    export interface IDestroyHandler {
        /** Specifies the reason for the destroy. */
        type: "left" | "canceled";
    }
    /**
     * Marks the destroy function that is invoked when the block is destroyed.
     * The function receives a `IDestroyHandler` object.
     */
    export function destroy<T extends NodeBlock>(
        pTarget: T,
        sFunction: keyof T,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "tripetto-runner-foundation/lib/fingerprint" {
    import { Slot } from "@tripetto/slots";
    import { IDefinition } from "@tripetto/map";
    import { Branch } from "tripetto-runner-foundation/lib/branches/branch";
    import { Cluster } from "tripetto-runner-foundation/lib/clusters/cluster";
    import { Condition } from "tripetto-runner-foundation/lib/conditions/condition";
    import { Node } from "tripetto-runner-foundation/lib/nodes/node";
    /** Generates data for the fingerprint. */
    export const fingerprintGenerator: (
        pRef: Branch | Cluster | Node | Condition | Slot
    ) => string;
    /** Validates nodes in the stencil. */
    export const stencilValidator: (pNode: Node) => boolean;
    /**
     * Calculates the fingerprint and stencil for the definition.
     * @param definition Specifies the definition.
     * @return Returns the fingerprint and stencil.
     */
    export function calculateFingerprintAndStencil(
        definition: IDefinition
    ): {
        readonly fingerprint: string;
        readonly stencil: (type: "exportables" | "actionables") => string;
    };
    /**
     * Calculates the fingerprint for the given definition.
     * @param definition Specifies the definition.
     * @return Returns the fingerprint hash.
     */
    export function fingerprint(definition: IDefinition): string;
}

declare module "tripetto-runner-foundation/lib/data/stencil" {
    import { IDefinition } from "@tripetto/map";
    import { Context, Instance } from "@tripetto/vfsm";
    import { Export } from "tripetto-runner-foundation/lib/data/export";
    /**
     * Calculates the stencil for the given definition, instance, context, or exportables.
     * @param type Specifies the stencil type.
     * @param src Specifies the definition, instance, context or exportables.
     * @return Returns the stencil hash.
     */
    export function stencil(
        type: "exportables",
        src: IDefinition | Instance | Context | Export.IExportables
    ): String;
    /**
     * Calculates the stencil for the given definition, instance, context, or actionables.
     * @param type Specifies the stencil type.
     * @param src Specifies the definition, instance, context or actionables.
     * @return Returns the stencil hash.
     */
    export function stencil(
        type: "actionables",
        src: IDefinition | Instance | Context | Export.IActionables
    ): string;
}

declare module "tripetto-runner-foundation/lib/data/checksum" {
    import { Context, Instance } from "@tripetto/vfsm";
    import { Export } from "tripetto-runner-foundation/lib/data/export";
    /**
     * Calculates the checksum of the data.
     * @param input Specifies the input to use for the checksum calculation.
     * @param trusted Specifies if the source is trusted. When the source is not
     * trusted some components of the checksum are calulcated based on the supplied
     * data.
     */
    export function checksum(
        input:
            | Instance
            | Context
            | {
                  exportables: Export.IExportables;
                  actionables: Export.IActionables | undefined;
              },
        trusted: boolean
    ): string;
}

declare module "tripetto-runner-foundation/lib/data/pow" {
    import { Context, Instance } from "@tripetto/vfsm";
    import { Export } from "tripetto-runner-foundation/lib/data/export";
    /**
     * Finds a 64 bit nonce that solves the specified difficulty.
     * @param data Specifies the data to find the nonce for.
     * @param difficulty Specifies the difficulty.
     * @param id Optional identifier for the data.
     * @param length Specifies the nonce length (defaults to `16`).
     * @param timeout Specifies a timeout in milliseconds (disabled by default).
     * @param timestamp Specifies the timestamp to use (defaults to current time).
     * @param timestampResolution Specifies if the timestamp has seconds or milliseconds resolution.
     * @returns Returns the nonce.
     */
    export function powSolve(
        data:
            | Instance
            | Context
            | {
                  exportables: Export.IExportables;
                  actionables: Export.IActionables | undefined;
              },
        difficulty: number,
        id?: string,
        length?: 8 | 16 | 24 | 32,
        timeout?: number,
        timestamp?: number,
        timestampResolution?: "seconds" | "milliseconds"
    ): string;
    /**
     * Verifies the supplied nonce.
     * @param nonce Specifies the nonce to verify.
     * @param difficulty Specifies the nonce difficulty.
     * @param validity Specifies the maximum validity period in milliseconds (or `0` to disable).
     * @param data Specifies the data for the nonce.
     * @param id Optional identifier for the data.
     */
    export function powVerify(
        nonce: string,
        difficulty: number,
        validity: number,
        data:
            | Instance
            | Context
            | {
                  exportables: Export.IExportables;
                  actionables: Export.IActionables | undefined;
              },
        id?: string
    ): boolean;
    /** Returns the hash rate in kH/s (only available after a nonce is generated). */
    export function powHashRate(): number;
    /** Returns the total amount of milliseconds spent on work. */
    export function powSpentTime(): number;
    /** Returns the duration in milliseconds of the last work. */
    export function powDuration(): number;
}

declare module "tripetto-runner-foundation/lib/storyline/orchestrator" {
    /** Dependencies */
    import { Instance } from "@tripetto/vfsm";
    import { IPrologue } from "@tripetto/map";
    import { IEpilogueContext } from "tripetto-runner-foundation/lib/epilogue";
    import { TModes } from "tripetto-runner-foundation/lib/modes";
    export interface IStorylineOrchestrator {
        /** Instance of the storyline. */
        readonly instance: Instance;
        /** Contains the mode of operation. */
        readonly mode: TModes;
        /** Retrieves the checksum. */
        readonly checksum: string | undefined;
        /** Retrieves the direction. */
        readonly direction: "forward" | "backward";
        /** Retrieves the prologue (if any). */
        readonly prologue: IPrologue | undefined;
        /** Retrieves the epilogue (if any). */
        readonly epilogue: IEpilogueContext;
    }
}

declare module "tripetto-runner-foundation/lib/conditions/blocks/decorator" {
    export interface IConditionBlockDecorator {
        /** Specifies a namespace for the block */
        readonly namespace?: string;
        /** Specifies this block is a condition block. */
        readonly type: "condition";
        /** Specifies the block identifier. */
        readonly identifier: string;
        /** Specifies optional type aliases. */
        readonly alias?: string | string[];
    }
}

declare module "tripetto-runner-foundation/lib/nodes/blocks/decorator" {
    export interface INodeBlockDecorator {
        /** Specifies a namespace for the block */
        readonly namespace?: string;
        /** Specifies this block is a node block. */
        readonly type: "node";
        /** Specifies the type identifier. */
        readonly identifier: string;
        /** Specifies optional type aliases. */
        readonly alias?: string | string[];
        /** Optional reference to something. */
        readonly ref?: {};
        /** Specifies if the block needs to be rerendered upon value store (disabled by default). */
        readonly autoRender?: boolean;
        /** Specifies if the block checks if all required slots contain a value (enabled by default). */
        readonly autoValidate?: boolean;
    }
    export interface IHeadlessBlockDecorator {
        /** Specifies a namespace for the block */
        readonly namespace?: string;
        /** Specifies this block is a headless block. */
        readonly type: "headless";
        /** Specifies the type identifier. */
        readonly identifier: string;
        /** Specifies optional type aliases. */
        readonly alias?: string | string[];
    }
}
