/*! Tripetto Runner Foundation 5.0.0 - Copyright (C) 2021 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/aes" {
    /** Explicit exports */
    export { encrypt, decrypt } from "@marblecore/aes/lib/aes";
}

declare module "@marblecore/aes/lib/aes" {
    /**
     * Apply Rijndael algorithm to input.
     * @param input Specifies the input.
     * @param keySchedule Specifies the key schedule array.
     */
    export function cipher(input: number[], keySchedule: number[][]): number[];
    /**
     * Performs key expansion to generate a key schedule.
     * @param input Specifies the key.
     * @returns Returns the key schedule array.
     */
    export function keyExpansion(input: number[]): number[][];
    /**
     * Encode multi-byte unicode string to ANSI string.
     * @param input Specifies the input string.
     * @returns Returns the ANSI string.
     */
    export function encodeUTF8(input: string): string;
    /**
     * Decodes an ANSI string to multi-byte unicode string.
     * @param input Specifies the input string.
     * @returns Returns the unicode string.
     */
    export function decodeUTF8(input: string): string;
    /**
     * Encrypt string using AES encryption in counter mode.
     * @param data Specifies the data to encrypt.
     * @param password Specifies the password to generate the key.
     * @param bits Specifies the number of bits for the key.
     * @returns Returns the encrypted data.
     */
    export function encrypt(
        data: string,
        password: string,
        bits: 128 | 192 | 256,
        btoa: (input: string) => string
    ): string;
    /**
     * Decrypts data using AES in counter mode.
     * @param data Specifies the data to encrypt.
     * @param password Specifies the password to generate the key.
     * @param bits Specifies the number of bits for the key.
     * @returns Returns the decrypted data.
     */
    export function decrypt(
        data: string,
        password: string,
        nBits: 128 | 192 | 256,
        atob: (input: string) => string
    ): string;
}
