/*! Tripetto Runner Foundation 5.0.0 - Copyright (C) 2021 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/l10n" {
    /** Explicit exports */
    export { IDomain } from "@marblecore/l10n/lib/domain";
    export { Translation } from "@marblecore/l10n/lib/translation";
    export {
        Translations,
        Translations as Domain,
        CONTEXT_SEPARATOR,
    } from "@marblecore/l10n/lib/translations";
    export { TTranslation } from "@marblecore/l10n/lib/translation";
    export {
        DEFAULTS as LOCALE_DEFAULTS,
        ILocale,
        Locales,
    } from "@marblecore/l10n/lib/locales";
    export {
        gettext,
        dgettext,
        ngettext,
        dngettext,
        pgettext,
        dpgettext,
        npgettext,
        dnpgettext,
        _,
        _n,
    } from "@marblecore/l10n/lib/l10n";
    export { IPluralRule, Plurals } from "@marblecore/l10n/lib/plurals";
    export { Namespace } from "@marblecore/l10n/lib/namespace";
}

declare module "@marblecore/l10n/lib/domain" {
    export interface IDomain {
        /** Specifies the locale (language[_territory]). */
        readonly locale: string;
        /** Specifies the language in english. */
        readonly language: string;
        /** Specifies the language in the native language. */
        readonly native: string;
    }
}

declare module "@marblecore/l10n/lib/translation" {
    export type TTranslation = {
        "": {
            readonly language?: string;
            readonly "plural-forms"?: string;
            readonly "plural-family"?: string;
        };
    } & {
        [id: string]: [null | string, ...string[]];
    };
    export class Translation {
        /**
         * Constructs a new domain.
         * @param pTranslation Reference to the translation data.
         */
        constructor(pTranslation: TTranslation);
        /** Retrieves the number of plurals. */
        get plurals(): number;
        /**
         * Enriches the domains with more translations.
         * @param pTranslation Reference to the translation.
         * @param bOverwrite Specifies if translations should be overwritten.
         */
        enrich(pTranslation: TTranslation, bOverwrite: boolean): void;
        /**
         * Retrieves the plural index for the supplied count.
         * @param nCount Specifies the count.
         * @return Returns the plural index.
         */
        plural(nCount: number): number;
        /**
         * Retrieves the translation for the specified message identifier or string.
         * @param sId Specifies the message identifier or string to retrieve.
         * @return Returns an array with translation strings.
         */
        translation(sId: string): [null | string, ...string[]] | undefined;
    }
}

declare module "@marblecore/l10n/lib/translations" {
    import { TTranslation } from "@marblecore/l10n/lib/translation";
    export const CONTEXT_SEPARATOR = "\u0004";
    export const DOMAIN_SEPARATOR = "\u0005";
    export const ARGUMENTS_SEPARATOR = "\u0006";
    export class Translations {
        /** Retrieves a list of loaded domains. */
        static get domains(): string[];
        /** Retrieves the default language. */
        static get default(): string;
        /** Sets the default domain. */
        static set default(sDomain: string);
        /** Retrieves the current domain. */
        static get current(): string;
        /** Sets the current domain. */
        static set current(sDomain: string);
        /**
         * Loads translation domains.
         * @param pTranslation Specifies the translation domains.
         * @param bMakeCurrent Specifies if the translation domain should be selected as current domain.
         * @param sMode Specifies if the translation domain needs to be enriched, overwritten or completely replaced.
         * @return Returns `true` if the domain is loaded.
         */
        static load(
            pTranslation: TTranslation | TTranslation[],
            bMakeCurrent?: boolean | "only-when-no-domain",
            sMode?: "enrich" | "overwrite" | "replace"
        ): boolean;
        /**
         * Unloads translations.
         * @param sDomain Specifies the domain to unload. If omitted all domains are unloaded.
         */
        static unload(sDomain?: string): void;
        /**
         * Verifies if the specified domain is loaded.
         * @param sDomain Specifies the language domain identifier.
         */
        static isLoaded(sDomain: string): boolean;
        /**
         * Resets all translations and activate the default domain.
         * @param sCurrent Specifies the language to reset the current domain to.
         */
        static reset(sCurrent?: string): void;
        /**
         * Translates the specified message.
         * @param sDomain Specifies the language domain identifier.
         * @param sContext Specifies the context of the message.
         * @param sId Specifies the message identifier or string to translate.
         * @param nCount Specifies the count for the plural.
         * @param bPlural Specifies if the message is a plural string.
         * @param sPluralId Specifies the plural message identifier or string to translate.
         * @param pArguments Specifies an array with string arguments which can be referenced in the context string using the percent sign
         * followed by the argument index `%n`. The first argument is referenced with `%1`.
         * @return Returns the formatted string.
         */
        static translate(
            sDomain: string,
            sContext: string,
            sId: string,
            bPlural?: boolean,
            sPluralId?: string,
            nCount?: number,
            ...pArguments: string[]
        ): string;
        /** Retrieves a list of loaded domains. */
        get domains(): string[];
        /** Retrieves the default language. */
        get default(): string;
        /** Sets the default domain. */
        set default(sDomain: string);
        /** Retrieves the current domain. */
        get current(): string;
        /** Sets the current domain. */
        set current(sDomain: string);
        /**
         * Loads translation domains.
         * @param pTranslation Specifies the translation domains.
         * @param bMakeCurrent Specifies if the translation domain should be selected as current domain.
         * @param sMode Specifies if the translation domain needs to be enriched, overwritten or completely replaced.
         * @return Returns `true` if the domain is loaded.
         */
        load(
            pTranslation: TTranslation | TTranslation[],
            bMakeCurrent: boolean | "only-when-no-domain",
            sMode: "enrich" | "overwrite" | "replace"
        ): boolean;
        /**
         * Unloads translations.
         * @param sDomain Specifies the domain to unload. If omitted all domains are unloaded.
         */
        unload(sDomain?: string): void;
        /**
         * Verifies if the specified domain is loaded.
         * @param sDomain Specifies the language domain identifier.
         */
        isLoaded(sDomain: string): boolean;
        /**
         * Resets all translations and activate the default domain.
         * @param sCurrent Specifies the language to reset the current domain to.
         */
        reset(sCurrent?: string): void;
        /**
         * Translates the specified message.
         * @param sDomain Specifies the language domain identifier.
         * @param sContext Specifies the context of the message.
         * @param sId Specifies the message identifier or string to translate.
         * @param nCount Specifies the count for the plural.
         * @param bPlural Specifies if the message is a plural string.
         * @param sPluralId Specifies the plural message identifier or string to translate.
         * @param pArguments Specifies an array with string arguments which can be referenced in the context string using the percent sign
         * followed by the argument index `%n`. The first argument is referenced with `%1`.
         * @return Returns the formatted string.
         */
        translate(
            sDomain: string,
            sContext: string,
            sId: string,
            bPlural: boolean,
            sPluralId: string,
            nCount: number,
            ...pArguments: string[]
        ): string;
    }
}

declare module "@marblecore/l10n/lib/locales" {
    import { IDomain } from "@marblecore/l10n/lib/domain";
    /** Describes a locale string. */
    export interface ILocaleString {
        readonly full: string;
        readonly long: string;
        readonly medium: string;
        readonly short: string;
    }
    /**
     * Describes the locale formats.
     * @param T Specifies the formatting type.
     */
    export interface ILocaleFormatting<T> {
        readonly formatted: T;
        readonly nominative: T;
    }
    /** Describes a locale number. */
    export interface ILocaleNumber {
        readonly decimals: string;
        readonly grouping: string;
        readonly minus: string;
    }
    /** Describes the locale months. */
    export interface ILocaleMonths {
        readonly abbreviated: string[];
        readonly narrow: string[];
        readonly wide: string[];
    }
    /** Describes the locale days. */
    export interface ILocaleDays {
        readonly abbreviated: string[];
        readonly narrow: string[];
        readonly short: string[];
        readonly wide: string[];
    }
    /** Describes the locale time. */
    export interface ILocaleTime {
        readonly AM: string;
        readonly PM: string;
    }
    /** Describes the locale translations. */
    export interface ILocaleTranslations {
        readonly months: ILocaleFormatting<ILocaleMonths>;
        readonly days: ILocaleFormatting<ILocaleDays>;
        readonly time: ILocaleTime;
    }
    /**  Describes the locale formats. */
    export interface ILocaleFormats {
        readonly date: ILocaleString;
        readonly time: ILocaleString;
        readonly dateTime: ILocaleString;
        readonly numbers: ILocaleNumber;
    }
    /**  Describes a locale. */
    export interface ILocale {
        readonly locale: string;
        readonly domain: string;
        readonly countryCode: string;
        readonly country: string;
        readonly countryNative: string;
        readonly language: string;
        readonly languageNative: string;
        readonly translations: ILocaleTranslations;
        readonly formats: ILocaleFormats;
    }
    /** Contains the default locale settings. */
    export const DEFAULTS: ILocale;
    export class Locales {
        /** Retrieves the locale information for the current locale. */
        static get locale(): ILocale;
        /** Retrieves the locale identifier string. */
        static get identifier(): string;
        /** Retrieves the domain for the current locale. */
        static get domain(): string;
        /** Retrieves the country code for the active locale. */
        static get countryCode(): string;
        /** Retrieves the country for the active locale. */
        static get country(): string;
        /** Retrieves the native country name for the active locale. */
        static get countryNative(): string;
        /** Retrieves the language for the active locale. */
        static get language(): string;
        /** Retrieves the language in the native language for the active locale. */
        static get languageNative(): string;
        /** Retrieves a list of locales. */
        static get locales(): string[];
        /** Retrieves a list of domains. Each domain contains the locale, language and native language string. */
        static get domains(): IDomain[];
        /**
         * Verifies if the specified locale is loaded.
         * @param sLocale Specifies the locale identifier.
         * @return Returns `true` if the locale is loaded.
         */
        static isLoaded(sLocale: string): boolean;
        /**
         * Loads the specified locale profiles.
         * @param pLocale Specifies the locale profile (multiple profiles can be supplied).
         * @param bSelect Specifies if the locale needs to be selected as active
         * locale (default is `true`, but only when a single locale is supplied).
         * @return Returns `true` if the locale data is loaded.
         */
        static load(pLocale: ILocale | ILocale[], bSelect?: boolean): boolean;
        /**
         * Retrieves the specified locale (or the current locale if no locale identifier is specified).
         * @param sLocale Specifies the locale identifier.
         * @return Returns the locale.
         */
        static get(sLocale?: string): ILocale;
        /**
         * Sets the current locale.
         * @param sLocale Specifies the locale identifier.
         * @return Returns `true` if the locale is set.
         */
        static set(sLocale: string): boolean;
        /**
         * Tries to find the most significant domain (ignoring case).
         * @param sDomain Specifies the domain.
         * @return Returns the domain string.
         */
        static findDomain(sDomain: string | undefined): string;
        /**
         * Retrieves domain information for the specified locale (or the current locale if no locale identifier is specified).
         * @param sLocale Specifies the locale identifier.
         * @return Returns the locale.
         */
        static getDomain(sLocale?: string): IDomain;
        /**
         * Formats a number by inserting thousands and decimal separators while taking rounding into account.
         * @param nValue Input value as a number or string.
         * @param nPrecision Specifies the precision (set it to `auto` to allow floating point numbers with automatic precision detection).
         * @param bThousands Specifies if thousands separators should be inserted.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted number as a string.
         */
        static number(
            nValue: number | string,
            nPrecision?: number | "auto",
            bThousands?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a date/time string.
         * @param sFormat Specifies the format to use, using the CLDR date/time formatting style. The following variables are available:
         * - `y` / `yyyy`: Specifies the full 4-digit year;
         * - `yy`: Specifies the 2-digit year;
         * - `M`: Specifies the month number;
         * - `MM`: Specifies the month number with a minimum of 2 digits;
         * - `MMM`: Specifies the month as an abbreviated string which should be supplied through the `pLocalesMonthFormattedAbbreviated`
         * parameter (e.g. `Jan`);
         * - `MMMM`: Specifies the month as a full string which should be supplied through the `pLocalesMonthFormattedWide` parameter (e.g.
         * `January`);
         * - `MMMMM`: Specifies the month as a narrow string which should be supplied through the `pLocalesMonthFormattedNarrow`
         * parameter (e.g. `J`);
         * - `LLL`: Specifies the month as an abbreviated nominative string which should be supplied through the
         * `pLocalesMonthNominativeAbbreviated` parameter (e.g. `Jan`);
         * - `LLLL`: Specifies the month as a full nominative string which should be supplied through the `pLocalesMonthNominativeWide`
         * parameter (e.g. `January`);
         * - `LLLLL`: Specifies the month as a narrow nominative string which should be supplied through the `pLocalesMonthNominativeNarrow`
         * parameter (e.g. `J`);
         * - `d`: Specifies the day of the month;
         * - `dd`: Specifies the day of the month with a minimum of 2 digits;
         * - `E`: Specifies the day of the week;
         * - `EE`: Specifies the day of the week as a string which should be supplied through the `pLocalesWeekdayFormattedShort`
         * parameter (e.g. `Su`);
         * - `EEE`: Specifies the day of the week as an abbreviated string which should be supplied through the
         * `pLocalesWeekdayFormattedAbbreviated` parameter (e.g. `Sun`);
         * - `EEEE`: Specifies the day of the week as a full string which should be supplied through the `pLocalesWeekdayFormattedWide`
         * parameter (e.g. `Sunday`);
         * - `EEEEE`: Specifies the day of the week as a narrow string which should be supplied through the `pLocalesWeekdayFormattedNarrow`
         * parameter (e.g. `S`);
         * - `cc`: Specifies the day of the week as a nominative string which should be supplied through the `pLocalesWeekdayNominativeShort`
         * parameter (e.g. `Su`);
         * - `ccc`: Specifies the day of the week as an abbreviated nominative string which should be supplied through the
         * `pLocalesWeekdayNominativeAbbreviated` parameter (e.g. `Sun`);
         * - `cccc`: Specifies the day of the week as a full nominative string which should be supplied through the
         * `pLocalesWeekdayNominativeWide` parameter (e.g. `Sunday`);
         * - `ccccc`: Specifies the day of the week as a narrow nominative string which should be supplied through the
         * `pLocalesWeekdayNominativeNarrow` parameter (e.g. `S`);
         * - `H`: Specifies the number of hours in 24-hour format;
         * - `HH`: Specifies the number of hours in 24-hour format  with a minimum of 2 digits;
         * - `h`: Specifies the number of hours in 12-hour format;
         * - `hh`: Specifies the number of hours in 12-hour format  with a minimum of 2 digits;
         * - `a`: Specifies the `AM` or `PM` hour suffix;
         * - `m`: Specifies the number of minutes;
         * - `mm`: Specifies the number of minutes with a minimum of 2 digits;
         * - `s`: Specifies the number of seconds;
         * - `ss`: Specifies the number of seconds with a minimum of 2 digits;
         * - `S`: Specifies the number of milliseconds;
         * - `SSS`: Specifies the number of milliseconds with a minimum of 3 digits;
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date string.
         * @see http://cldr.unicode.org/translation/date-time
         */
        static dateTime(
            sFormat: string,
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a full date string.
         * @param nTime Specifies the date to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date string.
         */
        static dateFull(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a long date string.
         * @param nTime Specifies the date to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date string.
         */
        static dateLong(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a medium date string.
         * @param nTime Specifies the date to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date string.
         */
        static dateMedium(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a short date string.
         * @param nTime Specifies the date to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date string.
         */
        static dateShort(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a full time string.
         * @param nTime Specifies the time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted time string.
         */
        static timeFull(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a long time string.
         * @param nTime Specifies the time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted time string.
         */
        static timeLong(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a medium time string.
         * @param nTime Specifies the time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted time string.
         */
        static timeMedium(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a short time string.
         * @param nTime Specifies the time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted time string.
         */
        static timeShort(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a full date/time string.
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date/time string.
         */
        static dateTimeFull(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a long date/time string.
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date/time string.
         */
        static dateTimeLong(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a medium date/time string.
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date/time string.
         */
        static dateTimeMedium(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /**
         * Formats a short date/time string.
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date/time string.
         */
        static dateTimeShort(
            nTime?: number,
            bUTC?: boolean,
            sLocale?: string
        ): string;
        /** Retrieves the locale information for the current locale. */
        get locale(): ILocale;
        /** Retrieves the locale identifier string. */
        get identifier(): string;
        /** Retrieves the domain for the current locale. */
        get domain(): string;
        /** Retrieves the country code for the active locale. */
        get countryCode(): string;
        /** Retrieves the country for the active locale. */
        get country(): string;
        /** Retrieves the native country name for the active locale. */
        get countryNative(): string;
        /** Retrieves the language for the active locale. */
        get language(): string;
        /** Retrieves the language in the native language for the active locale. */
        get languageNative(): string;
        /**
         * Loads the specified locale profile and activates it.
         * @param pLocale Specifies the locale profile (multiple profiles can be supplied).
         * @param bSelect Specifies if the locale needs to be selected as active
         * locale (default is `true`, but only when a single locale is supplied).
         * @return Returns `true` if the locale data is loaded.
         */
        load(pLocale: ILocale): boolean;
        /**
         * Sets the current locale.
         * @param sLocale Specifies the locale identifier.
         * @return Returns `true` if the locale is set.
         */
        set(sLocale: string): boolean;
        /**
         * Formats a number by inserting thousands and decimal separators while taking rounding into account.
         * @param nValue Input value as a number or string.
         * @param nPrecision Specifies the precision (set it to `auto` to allow floating point numbers with automatic precision detection).
         * @param bThousands Specifies if thousands separators should be inserted.
         * @return Returns the formatted number as a string.
         */
        number(
            nValue: number | string,
            nPrecision?: number | "auto",
            bThousands?: boolean
        ): string;
        /**
         * Formats a date/time string.
         * @param sFormat Specifies the format to use, using the CLDR date/time formatting style. The following variables are available:
         * - `y` / `yyyy`: Specifies the full 4-digit year;
         * - `yy`: Specifies the 2-digit year;
         * - `M`: Specifies the month number;
         * - `MM`: Specifies the month number with a minimum of 2 digits;
         * - `MMM`: Specifies the month as an abbreviated string which should be supplied through the `pLocalesMonthFormattedAbbreviated`
         * parameter (e.g. `Jan`);
         * - `MMMM`: Specifies the month as a full string which should be supplied through the `pLocalesMonthFormattedWide` parameter (e.g.
         * `January`);
         * - `MMMMM`: Specifies the month as a narrow string which should be supplied through the `pLocalesMonthFormattedNarrow`
         * parameter (e.g. `J`);
         * - `LLL`: Specifies the month as an abbreviated nominative string which should be supplied through the
         * `pLocalesMonthNominativeAbbreviated` parameter (e.g. `Jan`);
         * - `LLLL`: Specifies the month as a full nominative string which should be supplied through the `pLocalesMonthNominativeWide`
         * parameter (e.g. `January`);
         * - `LLLLL`: Specifies the month as a narrow nominative string which should be supplied through the `pLocalesMonthNominativeNarrow`
         * parameter (e.g. `J`);
         * - `d`: Specifies the day of the month;
         * - `dd`: Specifies the day of the month with a minimum of 2 digits;
         * - `E`: Specifies the day of the week;
         * - `EE`: Specifies the day of the week as a string which should be supplied through the `pLocalesWeekdayFormattedShort`
         * parameter (e.g. `Su`);
         * - `EEE`: Specifies the day of the week as an abbreviated string which should be supplied through the
         * `pLocalesWeekdayFormattedAbbreviated` parameter (e.g. `Sun`);
         * - `EEEE`: Specifies the day of the week as a full string which should be supplied through the `pLocalesWeekdayFormattedWide`
         * parameter (e.g. `Sunday`);
         * - `EEEEE`: Specifies the day of the week as a narrow string which should be supplied through the `pLocalesWeekdayFormattedNarrow`
         * parameter (e.g. `S`);
         * - `cc`: Specifies the day of the week as a nominative string which should be supplied through the `pLocalesWeekdayNominativeShort`
         * parameter (e.g. `Su`);
         * - `ccc`: Specifies the day of the week as an abbreviated nominative string which should be supplied through the
         * `pLocalesWeekdayNominativeAbbreviated` parameter (e.g. `Sun`);
         * - `cccc`: Specifies the day of the week as a full nominative string which should be supplied through the
         * `pLocalesWeekdayNominativeWide` parameter (e.g. `Sunday`);
         * - `ccccc`: Specifies the day of the week as a narrow nominative string which should be supplied through the
         * `pLocalesWeekdayNominativeNarrow` parameter (e.g. `S`);
         * - `H`: Specifies the number of hours in 24-hour format;
         * - `HH`: Specifies the number of hours in 24-hour format  with a minimum of 2 digits;
         * - `h`: Specifies the number of hours in 12-hour format;
         * - `hh`: Specifies the number of hours in 12-hour format  with a minimum of 2 digits;
         * - `a`: Specifies the `AM` or `PM` hour suffix;
         * - `m`: Specifies the number of minutes;
         * - `mm`: Specifies the number of minutes with a minimum of 2 digits;
         * - `s`: Specifies the number of seconds;
         * - `ss`: Specifies the number of seconds with a minimum of 2 digits;
         * - `S`: Specifies the number of milliseconds;
         * - `SSS`: Specifies the number of milliseconds with a minimum of 3 digits;
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @return Returns the formatted date string.
         * @see http://cldr.unicode.org/translation/date-time
         */
        dateTime(sFormat: string, nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a full date string.
         * @param nTime Specifies the date to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @return Returns the formatted date string.
         */
        dateFull(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a long date string.
         * @param nTime Specifies the date to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date string.
         */
        dateLong(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a medium date string.
         * @param nTime Specifies the date to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date string.
         */
        dateMedium(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a short date string.
         * @param nTime Specifies the date to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date string.
         */
        dateShort(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a full time string.
         * @param nTime Specifies the time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted time string.
         */
        timeFull(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a long time string.
         * @param nTime Specifies the time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted time string.
         */
        timeLong(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a medium time string.
         * @param nTime Specifies the time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted time string.
         */
        timeMedium(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a short time string.
         * @param nTime Specifies the time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted time string.
         */
        timeShort(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a full date/time string.
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date/time string.
         */
        dateTimeFull(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a long date/time string.
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date/time string.
         */
        dateTimeLong(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a medium date/time string.
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date/time string.
         */
        dateTimeMedium(nTime?: number, bUTC?: boolean): string;
        /**
         * Formats a short date/time string.
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param sLocale Specifies the locale to use.
         * @return Returns the formatted date/time string.
         */
        dateTimeShort(nTime?: number, bUTC?: boolean): string;
    }
}

declare module "@marblecore/l10n/lib/l10n" {
    /**
     * Translates a string using the default language.
     * @param sId Specifies the message identifier or string to translate.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The first argument is referenced with `%1`.
     * @return Returns the translated string.
     */
    export function gettext(sId: string, ...pArguments: string[]): string;
    /**
     * Translates a string using the specified language domain.
     * @param sDomain Specifies the language domain identifier.
     * @param sId Specifies the message identifier or string to translate.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The first argument is referenced with `%1`.
     * @return Returns the translated string.
     */
    export function dgettext(
        sDomain: string,
        sId: string,
        ...pArguments: string[]
    ): string;
    /**
     * Translates a plural string using the default language.
     * @param sId Specifies the message identifier or string to translate.
     * @param sPluralId Specifies the plural message identifier or string to translate.
     * @param nCount Specifies the count for the plural.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
     * @return Returns the translated string.
     */
    export function ngettext(
        sId: string,
        sPluralId: string,
        nCount: number,
        ...pArguments: string[]
    ): string;
    /**
     * Translates a plural string using the specified language domain.
     * @param sDomain Specifies the language domain identifier.
     * @param sId Specifies the message identifier or string to translate.
     * @param sPluralId Specifies the plural message identifier or string to translate.
     * @param nCount Specifies the count for the plural.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
     * @return Returns the translated string.
     */
    export function dngettext(
        sDomain: string,
        sId: string,
        sPluralId: string,
        nCount: number,
        ...pArguments: string[]
    ): string;
    /**
     * Translates a string from a specific context using the default language.
     * @param sContext Specifies the translation context.
     * @param sId Specifies the message identifier or string to translate.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The first argument is referenced with `%1`.
     * @return Returns the translated string.
     */
    export function pgettext(
        sContext: string,
        sId: string,
        ...pArguments: string[]
    ): string;
    /**
     * Translates a string from a specific context using the specified language domain.
     * @param sDomain Specifies the language domain identifier.
     * @param sContext Specifies the translation context.
     * @param sId Specifies the message identifier or string to translate.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The first argument is referenced with `%1`.
     * @return Returns the translated string.
     */
    export function dpgettext(
        sDomain: string,
        sContext: string,
        sId: string,
        ...pArguments: string[]
    ): string;
    /**
     * Translates a plural string using the default language.
     * @param sContext Specifies the translation context.
     * @param sId Specifies the message identifier or string to translate.
     * @param sPluralId Specifies the plural message identifier or string to translate.
     * @param nCount Specifies the count for the plural.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
     * @return Returns the translated string.
     */
    export function npgettext(
        sContext: string,
        sId: string,
        sPluralId: string,
        nCount: number,
        ...pArguments: string[]
    ): string;
    /**
     * Translates a plural string using the specified language domain.
     * @param sDomain Specifies the language domain identifier.
     * @param sContext Specifies the translation context.
     * @param sId Specifies the message identifier or string to translate.
     * @param sPluralId Specifies the plural message identifier or string to translate.
     * @param nCount Specifies the count for the plural.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
     * @return Returns the translated string.
     */
    export function dnpgettext(
        sDomain: string,
        sContext: string,
        sId: string,
        sPluralId: string,
        nCount: number,
        ...pArguments: string[]
    ): string;
    /**
     * Translates a string using the default language (short version for `gettext`).
     * @param sId Specifies the message identifier or string to translate.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
     * the argument index `%n`. The first argument is referenced with `%1`.
     * @return Returns the translated string.
     */
    export function _(sId: string, ...pArguments: string[]): string;
    /**
     * Translates a plural string using the default language (short version for `ngettext`).
     * @param sId Specifies the message identifier or string to translate.
     * @param sPluralId Specifies the plural message identifier or string to translate.
     * @param nCount Specifies the count for the plural.
     * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed
     * by the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
     * @return Returns the translated string.
     */
    export function _n(
        sId: string,
        sPluralId: string,
        nCount: number,
        ...pArguments: string[]
    ): string;
}

declare module "@marblecore/l10n/lib/plurals" {
    interface IPlural {
        readonly rules: string[];
        readonly expression: string;
        readonly locales: string[];
    }
    export interface IPluralRule extends IPlural {
        readonly family: string;
    }
    export class Plurals {
        /** Retrieves a list of plural rules. */
        static get rules(): IPluralRule[];
        /** Retrieves a list of locals with their plural rules. */
        static get locales(): {
            [locale: string]: IPluralRule | undefined;
        };
        /**
         * Retrieves the plural for the specified locale.
         * @param locale Specifies the local (if omitted the current locale will be used).
         * @return Returns the plural information or `undefined` if not found.
         */
        static getForLocale(locale?: string): IPluralRule | undefined;
        /**
         * Retrieves the plural for the specified family.
         * @param family Specifies the family.
         * @return Returns the plural information or `undefined` if not found.
         */
        static getForFamily(family: string): IPluralRule | undefined;
    }
    export {};
}

declare module "@marblecore/l10n/lib/namespace" {
    import { TTranslation } from "@marblecore/l10n/lib/translation";
    import { Locales } from "@marblecore/l10n/lib/locales";
    export class Namespace {
        /** Retrieves the global namespace (using the global translation and locale). */
        static get global(): Namespace;
        /** Creates a namespace with the specified identifier. */
        static create(identifier: string): Namespace;
        /** Retrieves a translation namespace with a global locale. */
        static translation(identifier: string): Namespace;
        /** Retrieves a reference to the locale instance. */
        get locale(): typeof Locales | Locales;
        /** Retrieves the default translation domain. */
        get default(): string;
        /** Sets the default translation domain. */
        set default(d: string);
        /** Retrieves the current translation domain. */
        get current(): string;
        /** Sets the current translation domain. */
        set current(c: string);
        /** Retrieves a list of loaded domains. */
        get domains(): string[];
        /**
         * Loads translation domains.
         * @param pTranslation Specifies the translation domains.
         * @param bMakeCurrent Specifies if the translation domain should be selected as current domain.
         * @param sMode Specifies if the translation domain needs to be enriched, overwritten or completely replaced.
         * @return Returns `true` if the domain is loaded.
         */
        get load(): (
            pTranslation: TTranslation | TTranslation[],
            bMakeCurrent?: boolean | "only-when-no-domain",
            sMode?: "enrich" | "overwrite" | "replace"
        ) => boolean;
        /**
         * Unloads translations.
         * @param sDomain Specifies the domain to unload. If omitted all domains are unloaded.
         */
        get unload(): (sDomain?: string) => void;
        /**
         * Verifies if the specified domain is loaded.
         * @param sDomain Specifies the language domain identifier.
         */
        get isLoaded(): (sDomain: string) => boolean;
        /**
         * Resets all translations and activate the default domain.
         * @param sCurrent Specifies the language to reset the current domain to.
         */
        get reset(): (sCurrent?: string) => void;
        /**
         * Translates a string using the default language.
         * @param sId Specifies the message identifier or string to translate.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The first argument is referenced with `%1`.
         * @return Returns the translated string.
         */
        get gettext(): (sId: string, ...pArguments: string[]) => string;
        /**
         * Translates a string using the specified language domain.
         * @param sDomain Specifies the language domain identifier.
         * @param sId Specifies the message identifier or string to translate.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The first argument is referenced with `%1`.
         * @return Returns the translated string.
         */
        get dgettext(): (
            sDomain: string,
            sId: string,
            ...pArguments: string[]
        ) => string;
        /**
         * Translates a plural string using the default language.
         * @param sId Specifies the message identifier or string to translate.
         * @param sPluralId Specifies the plural message identifier or string to translate.
         * @param nCount Specifies the count for the plural.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
         * @return Returns the translated string.
         */
        get ngettext(): (
            sId: string,
            sPluralId: string,
            nCount: number,
            ...pArguments: string[]
        ) => string;
        /**
         * Translates a plural string using the specified language domain.
         * @param sDomain Specifies the language domain identifier.
         * @param sId Specifies the message identifier or string to translate.
         * @param sPluralId Specifies the plural message identifier or string to translate.
         * @param nCount Specifies the count for the plural.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
         * @return Returns the translated string.
         */
        get dngettext(): (
            sDomain: string,
            sId: string,
            sPluralId: string,
            nCount: number,
            ...pArguments: string[]
        ) => string;
        /**
         * Translates a string from a specific context using the default language.
         * @param sContext Specifies the translation context.
         * @param sId Specifies the message identifier or string to translate.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The first argument is referenced with `%1`.
         * @return Returns the translated string.
         */
        get pgettext(): (
            sContext: string,
            sId: string,
            ...pArguments: string[]
        ) => string;
        /**
         * Translates a string from a specific context using the specified language domain.
         * @param sDomain Specifies the language domain identifier.
         * @param sContext Specifies the translation context.
         * @param sId Specifies the message identifier or string to translate.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The first argument is referenced with `%1`.
         * @return Returns the translated string.
         */
        get dpgettext(): (
            sDomain: string,
            sContext: string,
            sId: string,
            ...pArguments: string[]
        ) => string;
        /**
         * Translates a plural string using the default language.
         * @param sContext Specifies the translation context.
         * @param sId Specifies the message identifier or string to translate.
         * @param sPluralId Specifies the plural message identifier or string to translate.
         * @param nCount Specifies the count for the plural.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
         * @return Returns the translated string.
         */
        get npgettext(): (
            sContext: string,
            sId: string,
            sPluralId: string,
            nCount: number,
            ...pArguments: string[]
        ) => string;
        /**
         * Translates a plural string using the specified language domain.
         * @param sDomain Specifies the language domain identifier.
         * @param sContext Specifies the translation context.
         * @param sId Specifies the message identifier or string to translate.
         * @param sPluralId Specifies the plural message identifier or string to translate.
         * @param nCount Specifies the count for the plural.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
         * @return Returns the translated string.
         */
        get dnpgettext(): (
            sDomain: string,
            sContext: string,
            sId: string,
            sPluralId: string,
            nCount: number,
            ...pArguments: string[]
        ) => string;
        /**
         * Translates a string using the default language (short version for `gettext`).
         * @param sId Specifies the message identifier or string to translate.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed by
         * the argument index `%n`. The first argument is referenced with `%1`.
         * @return Returns the translated string.
         */
        get _(): (sId: string, ...pArguments: string[]) => string;
        /**
         * Translates a plural string using the default language (short version for `ngettext`).
         * @param sId Specifies the message identifier or string to translate.
         * @param sPluralId Specifies the plural message identifier or string to translate.
         * @param nCount Specifies the count for the plural.
         * @param pArguments Optional string arguments which can be referenced in the translation string using the percent sign followed
         * by the argument index `%n`. The count value `nCount` is automatically included as the first argument (`%1`).
         * @return Returns the translated string.
         */
        get _n(): (
            sId: string,
            sPluralId: string,
            nCount: number,
            ...pArguments: string[]
        ) => string;
    }
}
